@@ -17,6 +17,7 @@
 */
 
 #include "basics.h"
+#include "../config.h"
 
 #include <iostream>
 #include <limits>
@@ -28,6 +29,8 @@
 #ifdef __APPLE__
 	#include <sys/types.h>
 	#include <sys/sysctl.h>
+	#include <mach/mach.h>
+	#include <unistd.h>
 #elif defined __linux__
 	//Needed for getting ram total usage under linux
 	#include <sys/sysinfo.h>
@@ -40,9 +43,46 @@ using std::list;
 using std::cerr;
 using std::endl;
 
+//Name of the  DTD file for state loading
+const char *DTD_NAME="threeDepict-state.dtd";
+//Program name
+const char *PROGRAM_NAME = "3Depict";
+//Program version
+const char *PROGRAM_VERSION = "0.0.5";
+//Path to font for Default FTGL  font
+const char *FONT_FILE= "FreeSans.ttf";
+
+std::string boolStrEnc(bool b)
+{
+	if(b)
+		return "1";
+	else
+		return "0";
+}
 //default font to use.
 std::string defaultFontFile;
 
+std::string onlyFilename( const std::string& path) 
+{
+#if defined(_WIN32) || defined(_WIN64)
+	//windows uses "\" as path sep, just to be different
+	return path.substr( path.find_last_of( '\\' ) +1 );
+#else
+	//The entire world, including the interwebs, uses  "/" as the path sep.
+	return path.substr( path.find_last_of( '/' ) +1 );
+#endif
+}
+
+std::string onlyDir( const std::string& path) 
+{
+#if defined(_WIN32) || defined(_WIN64)
+	//windows uses "\" as path sep, just to be different
+	return path.substr(0, path.find_last_of( '\\' ) +1 );
+#else
+	//The entire world, including the interwebs, uses  "/" as the path sep.
+	return path.substr(0, path.find_last_of( '/' ) +1 );
+#endif
+}
 void setDefaultFontFile(const std::string &font)
 {
 	defaultFontFile=font;
@@ -66,12 +106,109 @@ void nullifyMarker(char *buffer, char marker)
 	}
 }
 
-void dh_assert(const char * const filename, const unsigned int lineNumber) 
+bool parsePointStr(const std::string &str,Point3D &pt)
 {
-    std::cerr << "ASSERTION ERROR!" << std::endl;
-    std::cerr << "Filename: " << filename << std::endl;
-    std::cerr << "Line number: " << lineNumber << std::endl;
+	//Needs to be at minimum #,#,#
+	if(str.size()< 5)
+		return false;
 
+	//GCC might bitch about this. Ignore gcc.
+	bool usingBrackets;
+	string tmpStr;
+	tmpStr=stripWhite(str);
+
+
+	//Two strings must be in sync
+	std::string allowableStartChars, allowableEndChars;
+	allowableStartChars="([{<'";
+	allowableEndChars=")]}>'";
+
+	unsigned int nSeparators=0;
+	unsigned int separatorPos[2];
+	//GCC might bitch about this. Ignore gcc.
+	unsigned int nStart,nEnd;
+
+	size_t startPos,endPos;
+	startPos=allowableStartChars.find(tmpStr[0]);
+	endPos=allowableEndChars.find(tmpStr[tmpStr.size()-1]);
+
+
+	std::string allowableSeparators=",; \t|_";
+
+	if(startPos ==endPos && startPos !=std::string::npos) 
+	{
+		//Could be:
+		//(#,#,#) format; where ( and ) are any of the allowable start/ends
+		if(str.size()< 7)
+			return false;
+		nStart=1;
+		nEnd=tmpStr.size()-1;
+		usingBrackets=true;
+	}
+	else if(startPos== std::string::npos && endPos == std::string::npos)
+	{
+		//Possible allowable format:
+		//(-)#,(-)#,(-)# format : brackets( ) denote optional
+		if(!(isdigit(tmpStr[0]) || tmpStr[0] == '-') || 
+			!isdigit(tmpStr[tmpStr.size()-1]))
+		       return false;
+
+		nStart=0;
+		nEnd=tmpStr.size();
+		usingBrackets=false;
+	}
+
+
+	size_t curSep=std::numeric_limits<size_t>::max();
+	for(unsigned int ui=nStart; ui<nEnd; ui++)
+	{
+		size_t separatorTypeIdx;
+		//Look through our list of allowable separators
+		separatorTypeIdx=allowableSeparators.find(tmpStr[ui]);
+		if(separatorTypeIdx != std::string::npos) 
+		{
+			if(curSep != std::numeric_limits<size_t>::max()) 
+			{
+				//Check we have been using the same separator each time
+				if(curSep != separatorTypeIdx)
+					return false;
+			}
+			else
+				curSep=separatorTypeIdx;
+
+			separatorPos[nSeparators]=ui;
+			nSeparators++;
+			if(nSeparators > 2)
+				return false;
+		}
+	}
+	if(nSeparators!=2)
+		return false;
+
+	unsigned int length;
+	if(usingBrackets)
+		length= separatorPos[0]-1;
+	else
+		length= separatorPos[0];
+	string tmpStrTwo;
+	tmpStrTwo =tmpStr.substr(nStart,length);
+	
+	float p[3];
+	if(stream_cast(p[0],tmpStrTwo))
+		return false;
+
+
+	tmpStrTwo =tmpStr.substr(separatorPos[0]+1,separatorPos[1]-(separatorPos[0]+1));
+	if(stream_cast(p[1],tmpStrTwo))
+		return false;
+	
+	tmpStrTwo =tmpStr.substr(separatorPos[1]+1,nEnd-(separatorPos[1]));
+	if(stream_cast(p[2],tmpStrTwo))
+		return false;
+
+	pt.setValueArr(p);
+
+	return true;
 }
 
 void ucharToHexStr(unsigned char c, std::string &s)
@@ -121,19 +258,57 @@ void hexStrToUChar(const std::string &s, unsigned char &c)
 	c = 16*high + low;
 }
 
-unsigned int genUniqueID(const vector<unsigned int> &vec)
+std::string convertFileStringToCanonical(const std::string &s)
 {
-	//Look for each element number as a unique value in turn
-	for(unsigned int ui=0;ui<vec.size(); ui++)
+	//We call unix the "canonical" format. 
+	//otherwise we substitute "\"s for "/"s
+#if (__WIN32) || (__WIN64)
+	string r;
+	for(unsigned int ui=0;ui<s.size();ui++)
 	{
-		if(std::find(vec.begin(),vec.end(),ui) == vec.end())
-			return ui;
+		if(s[ui] == '\\')
+			r+="/";
+		else
+			r+=s[ui];
 	}
+	return r;
+#else
+	return s;
+#endif
+}
 
-	//Note: This is guaranteed to be unique
-	//via the pigeonhole principle (as we found every element, and the sizes
-	//match, then the vector can only contain the elements we searched for)
-	return vec.size();
+std::string convertFileStringToNative(const std::string &s)
+{
+	//We call unix the "canonical" format. 
+	//otherwise we substitute "/"s for "\"s
+#if (__WIN32) || (__WIN64)
+	string r;
+	for(unsigned int ui=0;ui<s.size();ui++)
+	{
+		if(s[ui] == '/')
+			r+="\\";
+		else
+			r+=s[ui];
+	}
+	return r;
+#else
+	return s;
+#endif
+}
+
+std::string digitString(unsigned int thisDigit, unsigned int maxDigit)
+{
+	std::string s,thisStr;
+	stream_cast(thisStr,thisDigit);
+
+	stream_cast(s,maxDigit);
+	for(unsigned int ui=0;ui<s.size();ui++)
+		s[ui]='0';
+
+
+	s=s.substr(0,s.size()-thisStr.size());	
+
+	return  s+thisStr;
 }
 
 std::string choiceString(std::vector<std::pair<unsigned int, std::string> > comboString, 
@@ -162,9 +337,10 @@ std::string choiceString(std::vector<std::pair<unsigned int, std::string> > comb
 }
 
 //!Returns Choice ID from string (see choiceString(...) for string format)
+//FIXME: Does not work if the choicestring starts from a number other than zero...
 std::string getActiveChoice(std::string choiceString)
 {
-	unsigned int colonPos;
+	size_t colonPos;
 	colonPos = choiceString.find(":");
 	ASSERT(colonPos!=string::npos);
 
@@ -185,48 +361,29 @@ std::string getActiveChoice(std::string choiceString)
 	return tmpStr.substr(tmpStr.find("|")+1,tmpStr.size()-1);
 }
 
-//Convert my internal choice string format to wx's
-std::string wxChoiceParamString(std::string choiceString)
-{
-	string retStr;
 
-	bool haveColon=false;
-	bool haveBar=false;
-	for(unsigned int ui=0;ui<choiceString.size();ui++)
-	{
-		if(haveColon && haveBar )
-		{
-			if(choiceString[ui] != ',')
-				retStr+=choiceString[ui];
-			else
-			{
-				haveBar=false;
-				retStr+=",";
-			}
-		}
-		else
-		{
-			if(choiceString[ui]==':')
-				haveColon=true;
-			else
-			{
-				if(choiceString[ui]=='|')
-					haveBar=true;
-			}
-		}
-	}
-
-	return retStr;
-}
-
-void parseColString(const std::string &str,
+bool parseColString(const std::string &str,
 	unsigned char &r, unsigned char &g, unsigned char &b, unsigned char &a)
 {
 	//Input string is in 2 char hex form, 3 or 4 colour, with # leading. RGB order
 	//lowercase string.
-	ASSERT(str.size() == 9 || str.size() == 7);
+	if(str.size() != 9 && str.size() != 7)
+		return false;
 
-	ASSERT(str[0] == '#');
+	if(str[0] != '#')
+		return false;
+
+	string rS,gS,bS,aS;
+	rS=str.substr(1,2);
+	gS=str.substr(3,2);
+	bS=str.substr(5,2);
+
+	if(!isxdigit(rS[0]) || !isxdigit(rS[1]))
+		return false;
+	if(!isxdigit(gS[0]) || !isxdigit(gS[1]))
+		return false;
+	if(!isxdigit(bS[0]) || !isxdigit(bS[1]))
+		return false;
 
 	hexStrToUChar(str.substr(1,2),r);	
 	hexStrToUChar(str.substr(3,2),g);	
@@ -236,8 +393,12 @@ void parseColString(const std::string &str,
 		a = 255;
 	else
 	{
+		aS=str.substr(7,2);
+		if(!isxdigit(aS[0]) || !isxdigit(aS[1]))
+			return false;
 		hexStrToUChar(str.substr(7,2),a);	
 	}
+	return true;
 }
 
 void genColString(unsigned char r, unsigned char g, 
@@ -268,6 +429,7 @@ void genColString(unsigned char r, unsigned char g,
 	ucharToHexStr(b,tmp);
 	s+=tmp;
 }
+
 //Strip "whitespace"
 std::string stripWhite(const std::string &str)
 {
@@ -448,272 +610,6 @@ bool Colour::fromString(const std::string &str)
 }
 
 
-void Point3D::copyValueArr(float *valArr) const
-{
-	ASSERT(valArr);
-	//compiler should unroll this automatically
-	for(unsigned int ui=0; ui<3; ui++)
-	{
-		*(valArr+ui) = *(value+ui);
-	}
-}
-
-bool Point3D::operator==(const Point3D &pt) const
-{
-	return (value[0] == pt.value[0] && value[1] == pt.value[1] && value[2] == pt.value[2]);
-}
-
-Point3D Point3D::operator=(const Point3D &pt)
-{
-	value [0] = pt.value[0];
-	value [1] = pt.value[1];
-	value [2] = pt.value[2];
-	return *this;
-}
-
-Point3D Point3D::operator+=(const Point3D &pt)
-{
-	for(unsigned int ui=0;ui<3; ui++)
-		value[ui]+= pt.value[ui];
-	
-	return *this;
-}
-
-Point3D Point3D::operator+(const Point3D &pt) const
-{
-	Point3D ptTmp;
-	
-	for(unsigned int ui=0;ui<3; ui++)
-		ptTmp.value[ui] = value[ui]  + pt.value[ui];
-	
-	return ptTmp;
-}
-
-Point3D Point3D::operator+(float f) 
-{
-	for(unsigned int ui=0;ui<3; ui++)
-		value[ui] = value[ui]  + f;
-	
-	return *this;
-}
-
-Point3D Point3D::operator-(const Point3D &pt) const
-{
-	Point3D ptTmp;
-	
-	for(unsigned int ui=0;ui<3; ui++)
-		ptTmp.value[ui] = value[ui]  - pt.value[ui];
-	
-	return ptTmp;
-}
-
-Point3D Point3D::operator-() const
-{
-	Point3D ptTmp;
-	
-	for(unsigned int ui=0;ui<3; ui++)
-		ptTmp.value[ui] = -value[ui];
-	
-	return ptTmp;
-}
-
-Point3D Point3D::operator*=(const float scale)
-{
-	value[0] = value[0]*scale;
-	value[1] = value[1]*scale;
-	value[2] = value[2]*scale;
-	
-	return *this;
-}
-
-Point3D Point3D::operator*(float scale) const
-{
-	Point3D tmpPt;
-
-	tmpPt.value[0] = value[0]*scale;
-	tmpPt.value[1] = value[1]*scale;
-	tmpPt.value[2] = value[2]*scale;
-
-	return tmpPt;
-}
-
-Point3D Point3D::operator/(float scale) const
-{
-	Point3D tmpPt;
-
-	scale = 1.0f/scale;
-	tmpPt.value[0] = value[0]*scale;
-	tmpPt.value[1] = value[1]*scale;
-	tmpPt.value[2] = value[2]*scale;
-
-	return tmpPt;
-}
-
-float Point3D::sqrDist(const Point3D &pt) const
-{
-	return (pt.value[0]-value[0])*(pt.value[0]-value[0])+
-		(pt.value[1]-value[1])*(pt.value[1]-value[1])+
-		(pt.value[2]-value[2])*(pt.value[2]-value[2]);
-}
-		
-float Point3D::dotProd(const Point3D &pt) const
-{
-	//Return the inner product
-	return value[0]*pt.value[0] + value[1]*pt.value[1] + value[2]*pt.value[2];
-}
-
-Point3D Point3D::crossProd(const Point3D &pt) const
-{
-	Point3D cross;
-
-	cross.value[0] = (pt.value[2]*value[1] - pt.value[1]*value[2]);
-	cross.value[1] = -(value[0]*pt.value[2] - pt.value[0]*value[2]);
-	cross.value[2] = (value[0]*pt.value[1] - value[1]*pt.value[0]);
-
-	return cross;
-}
-
-bool Point3D::insideBox(const Point3D &farPoint) const
-{
-	
-	return (value[0] < farPoint.value[0] && value[0] >=0) &&
-		(value[1] < farPoint.value[1] && value[1] >=0) &&
-		(value[2] < farPoint.value[2] && value[2] >=0);
-}
-
-bool Point3D::insideBox(const Point3D &lowPt,const Point3D &highPt) const
-{
-	
-	return (value[0] < highPt.value[0] && value[0] >=lowPt.value[0]) &&
-		(value[1] < highPt.value[1] && value[1] >=lowPt.value[1]) &&
-		(value[2] < highPt.value[2] && value[2] >=lowPt.value[2]);
-}
-
-//This is different to +=, because it generates no return value
-void Point3D::add(const Point3D &obj)
-{
-	value[0] = obj.value[0] + value[0];
-	value[1] = obj.value[1] + value[1];
-	value[2] = obj.value[2] + value[2];
-}
-
-float Point3D::sqrMag() const
-{
-	return value[0]*value[0] + value[1]*value[1] + value[2]*value[2];
-}
-
-Point3D Point3D::normalise()
-{
-	float mag = sqrt(sqrMag());
-
-	value[0]/=mag;
-	value[1]/=mag;
-	value[2]/=mag;
-
-	return *this;
-}
-
-void Point3D::negate() 
-{
-	value[0] = -value[0];
-	value[1] = -value[1];
-	value[2] = -value[2];
-}
-
-float Point3D::angle(const Point3D &pt) const
-{
-	return acos(dotProd(pt)/(sqrt(sqrMag()*pt.sqrMag())));
-}
-
-#ifdef __LITTLE_ENDIAN__
-
-void Point3D::switchEndian()
-{
-	floatSwapBytes(&value[0]);
-	floatSwapBytes(&value[1]);
-	floatSwapBytes(&value[2]);
-}
-#endif
-
-std::ostream& operator<<(std::ostream &stream, const Point3D &pt)
-{
-	stream << "(" << pt.value[0] << "," << pt.value[1] << "," << pt.value[2] << ")";
-	return stream;
-}
-
-bool Point3D::parse(const std::string &str)
-{
-	//Needs to be at minimum #,#,#
-	if(str.size()< 5)
-		return false;
-
-	bool usingBrackets;
-	string tmpStr;
-	tmpStr=stripWhite(str);
-
-	unsigned int nCommas=0;
-	unsigned int commaPos[2];
-	unsigned int nStart,nEnd;
-	if( tmpStr[0] == '(' && tmpStr[tmpStr.size()-1] == ')')
-	{
-		//(#,#,#) format
-		if(str.size()< 7)
-			return false;
-		nStart=1;
-		nEnd=tmpStr.size()-1;
-		usingBrackets=true;
-	}
-	else
-	{
-		//(-)#,(-)#,(-)# format : brackets( ) denote optional
-		if(!(isdigit(tmpStr[0]) || tmpStr[0] == '-') || 
-			!isdigit(tmpStr[tmpStr.size()-1]))
-		       return false;
-
-		nStart=0;
-		nEnd=tmpStr.size();
-		usingBrackets=false;
-	}
-	for(unsigned int ui=nStart; ui<nEnd; ui++)
-	{
-		if(tmpStr[ui] == ',')
-		{
-			commaPos[nCommas]=ui;
-			nCommas++;
-			if(nCommas > 2)
-				return false;
-		}
-	}
-	if(nCommas!=2)
-		return false;
-
-	unsigned int length;
-	if(usingBrackets)
-		length= commaPos[0]-1;
-	else
-		length= commaPos[0];
-	string tmpStrTwo;
-	tmpStrTwo =tmpStr.substr(nStart,length);
-	
-	float p[3];
-	if(stream_cast(p[0],tmpStrTwo))
-		return false;
-
-
-	tmpStrTwo =tmpStr.substr(commaPos[0]+1,commaPos[1]-(commaPos[0]+1));
-	if(stream_cast(p[1],tmpStrTwo))
-		return false;
-	
-	tmpStrTwo =tmpStr.substr(commaPos[1]+1,nEnd-(commaPos[1]));
-	if(stream_cast(p[2],tmpStrTwo))
-		return false;
-
-	value[0]=p[0];
-	value[1]=p[1];
-	value[2]=p[2];
-
-	return true;
-}
 
 //========
 
@@ -721,14 +617,12 @@ void BoundCube::setBounds(const std::vector<Point3D> &points)
 {
 	
 	setInverseLimits();	
-#pragma omp parallel for
 	for(unsigned int ui=0; ui<points.size(); ui++)
 	{
 		for(unsigned int uj=0; uj<3; uj++)
 		{
 			if(points[ui].getValue(uj) < bounds[uj][0])
 			{
-				#pragma omp critical
 				{
 				bounds[uj][0] = points[ui].getValue(uj);
 				valid[uj][0]=true;
@@ -737,7 +631,6 @@ void BoundCube::setBounds(const std::vector<Point3D> &points)
 			
 			if(points[ui].getValue(uj) > bounds[uj][1])
 			{
-				#pragma omp critical
 				{
 				bounds[uj][1] = points[ui].getValue(uj);
 				valid[uj][1]=true;
@@ -746,16 +639,9 @@ void BoundCube::setBounds(const std::vector<Point3D> &points)
 		}
 	}
 
-	valid[0][0] =true;
-	valid[1][0] =true;
-	valid[2][0] =true;
-	
-	valid[0][1] =true;
-	valid[1][1] =true;
-	valid[2][1] =true;
-
 }
 
+
 void BoundCube::setInverseLimits()
 {
 	bounds[0][0] = std::numeric_limits<float>::max();
@@ -765,6 +651,14 @@ void BoundCube::setInverseLimits()
 	bounds[0][1] = -std::numeric_limits<float>::max();
 	bounds[1][1] = -std::numeric_limits<float>::max();
 	bounds[2][1] = -std::numeric_limits<float>::max();
+
+	valid[0][0] =false;
+	valid[1][0] =false;
+	valid[2][0] =false;
+	
+	valid[0][1] =false;
+	valid[1][1] =false;
+	valid[2][1] =false;
 }
 bool BoundCube::isValid() const
 {
@@ -773,9 +667,22 @@ bool BoundCube::isValid() const
 		if(!valid[ui][0] || !valid[ui][1])
 			return false;
 	}
+
 	return true;
 }
 
+bool BoundCube::isFlat() const
+{
+	//Test the limits being inverted or equated
+	for(unsigned int ui=0;ui<3; ui++)
+	{
+		if(fabs(bounds[ui][0] - bounds[ui][1]) < std::numeric_limits<float>::epsilon())
+			return true;
+	}	
+
+	return false;
+}
+
 void BoundCube::expand(const BoundCube &b) 
 {
 	//Check both lower and upper limit.
@@ -785,8 +692,8 @@ void BoundCube::expand(const BoundCube &b)
 		return;
 
 	//If self not valid, ensure that it will be after this run
-	if(!isValid())
-		setInverseLimits();
+	//if(!isValid())
+	//	setInverseLimits();
 
 	for(unsigned int ui=0; ui<3; ui++)
 	{
@@ -807,7 +714,7 @@ void BoundCube::expand(const BoundCube &b)
 void BoundCube::expand(const Point3D &p) 
 {
 	//If self not valid, ensure that it will be after this run
-	ASSERT(isValid())
+	//ASSERT(isValid())
 	for(unsigned int ui=0; ui<3; ui++)
 	{
 		//Check lower bound is lower to new pt
@@ -872,6 +779,25 @@ void BoundCube::getBounds(Point3D &low, Point3D &high) const
 	}
 }
 
+float BoundCube::getLargestDim() const
+{
+	float f;
+	f=getSize(0);
+	f=std::max(getSize(1),f);	
+	return std::max(getSize(2),f);	
+}
+
+bool BoundCube::containsPt(const Point3D &p) const
+{
+	for(unsigned int ui=0; ui<3; ui++)
+	{
+		ASSERT(valid[ui][0] && valid[ui][1]);
+		if(p.getValue(ui) < bounds[ui][0] || p.getValue(ui) > bounds[ui][1])
+			return false;
+	}
+	return true;
+}
+
 float BoundCube::getSize(unsigned int dim) const
 {
 	ASSERT(dim < 3);
@@ -884,48 +810,32 @@ float BoundCube::getSize(unsigned int dim) const
 	return fabs(bounds[dim][1] - bounds[dim][0]);
 }
 
-bool BoundCube::containsPt(const Point3D &p) const
-{
-	for(unsigned int ui=0; ui<3; ui++) 
-	{
-		ASSERT(valid[ui][0] && valid[ui][1]);
-		if( p[ui] > bounds[ui][0] && p[ui] < bounds[ui][1])
-			return true;
-	}
-
-	return false;
-}
-
 //checks intersection with sphere [centre,centre+radius)
 bool BoundCube::intersects(const Point3D &pt, float sqrRad)
 {
 	Point3D nearPt;
 	
 	//Find the closest point on the cube  to the sphere
-	unsigned int ui=2;
-	const float *val=pt.getValueArr()+ui;
-	do
+	for(unsigned int ui=0;ui<3;ui++)
 	{
-		if(*val <= bounds[ui][0])
+		if(pt.getValue(ui) <= bounds[ui][0])
 		{
 			nearPt.setValue(ui,bounds[ui][0]);
-			--val;
 			continue;
 		}
 		
-		if(*val >=bounds[ui][1])
+		if(pt.getValue(ui) >=bounds[ui][1])
 		{
 			nearPt.setValue(ui,bounds[ui][1]);
-			--val;
 			continue;
 		}
 	
-		nearPt.setValue(ui,*val);
-		--val;
-	}while(ui--);
+		nearPt.setValue(ui,pt[ui]);
+	}
 
 	//now test the distance from nrPt to pt
-	return (nearPt.sqrDist(pt) < sqrRad);
+	//Note that the touching case is considered to be an intersection
+	return (nearPt.sqrDist(pt) <=sqrRad);
 }
 
 Point3D BoundCube::getCentroid() const
@@ -967,10 +877,38 @@ float BoundCube::getMaxDistanceToBox(const Point3D &queryPt) const
 			queryPt.sqrDist(Point3D(p[ui&1][0],p[(ui&2) >> 1][1],p[(ui&4) >> 2][2])));
 	}
 
-	return sqrt(maxDistSqr);
+	return sqrtf(maxDistSqr);
 }
 
-BoundCube BoundCube::operator=(const BoundCube &b)
+bool BoundCube::containedInSphere(const Point3D &queryPt,float sqrDist) const
+{
+#ifdef DEBUG
+	for(unsigned int ui=0;ui<3; ui++)
+	{
+		ASSERT(valid[ui][1] && valid [ui][0]);
+	}
+#endif
+
+
+	//Could probably be a bit more compact.
+	
+
+	//Set lower and upper corners on the bounding rectangle
+	Point3D p[2];
+	p[0].setValue(bounds[0][0],bounds[1][0],bounds[2][0]);
+	p[1].setValue(bounds[0][1],bounds[1][1],bounds[2][1]);
+
+	//Count binary-wise selecting upper and lower limits, to enumerate all 8 verticies.
+	for(unsigned int ui=0;ui<9; ui++)
+	{
+		if(queryPt.sqrDist(Point3D(p[ui&1][0],p[(ui&2) >> 1][1],p[(ui&4) >> 2][2])) > sqrDist)
+			return false;
+	}
+
+	return true;
+}
+
+const BoundCube &BoundCube::operator=(const BoundCube &b)
 {
 	for(unsigned int ui=0;ui<3; ui++)
 	{
@@ -1031,7 +969,7 @@ unsigned int UniqueIDHandler::getPos(unsigned int id) const
 {
 
 	for(list<std::pair<unsigned int, unsigned int> >::const_iterator it=idList.begin();
-			it!=idList.end(); it++)
+			it!=idList.end(); ++it)
 	{
 		if(id == it->second)
 			return it->first;
@@ -1043,7 +981,7 @@ unsigned int UniqueIDHandler::getPos(unsigned int id) const
 void UniqueIDHandler::killByPos(unsigned int pos)
 {
 	for(list<std::pair<unsigned int, unsigned int> >::iterator it=idList.begin();
-			it!=idList.end(); it++)
+			it!=idList.end(); ++it)
 	{
 		if(pos  == it->first)
 		{
@@ -1054,7 +992,7 @@ void UniqueIDHandler::killByPos(unsigned int pos)
 
 	//Decreement the items, which were further along, in order to maintin the mapping	
 	for(list<std::pair<unsigned int, unsigned int> >::iterator it=idList.begin();
-			it!=idList.end(); it++)
+			it!=idList.end(); ++it)
 	{
 		if( it->first > pos)
 			it->first--;
@@ -1064,7 +1002,7 @@ void UniqueIDHandler::killByPos(unsigned int pos)
 unsigned int UniqueIDHandler::getId(unsigned int pos) const
 {
 	for(list<std::pair<unsigned int, unsigned int> >::const_iterator it=idList.begin();
-			it!=idList.end(); it++)
+			it!=idList.end(); ++it)
 	{
 		if(pos == it->first)
 			return it->second;
@@ -1085,7 +1023,7 @@ unsigned int UniqueIDHandler::genId(unsigned int pos)
 		bool idTaken;
 		idTaken=false;
 		for(list<std::pair<unsigned int, unsigned int> >::iterator it=idList.begin();
-				it!=idList.end(); it++)
+				it!=idList.end(); ++it)
 		{
 			if(ui == it->second)
 			{
@@ -1109,7 +1047,7 @@ void UniqueIDHandler::getIds(std::vector<unsigned int> &idVec) const
 {
 	//most wordy way of saying "spin through list" ever.
 	for(list<std::pair<unsigned int, unsigned int> >::const_iterator it=idList.begin();
-			it!=idList.end(); it++)
+			it!=idList.end(); ++it)
 		idVec.push_back(it->second);
 }
 
@@ -1135,7 +1073,7 @@ int getTotalRAM()
 	// Get RAM snapshot
 	::GlobalMemoryStatus(&MemStat);
 	ret= MemStat.dwTotalPhys / (1024*1024);
-#elif __APPLE__
+#elif __APPLE__ || __FreeBSD__
 
 	uint64_t mem;
 	size_t len = sizeof(mem);
@@ -1153,7 +1091,7 @@ int getTotalRAM()
     return ret;
 }
 
-int getAvailRAM()
+size_t getAvailRAM()
 {
     int ret = 0;
 #if defined(_WIN32) || defined(_WIN64) || defined(__CYGWIN__)   
@@ -1165,13 +1103,19 @@ int getAvailRAM()
 	::GlobalMemoryStatus(&MemStat);
 	ret= MemStat.dwAvailPhys / (1024*1024);
 
-#elif __APPLE__
-	uint64_t mem;
-	size_t len = sizeof(mem);
-
-	sysctlbyname("hw.usermem", &mem, &len, NULL, 0);
-
-	ret = (int)(mem/(1024*1024));
+#elif __APPLE__ || __FreeBSD__
+	uint64_t        memsize;
+	
+	size_t                  pagesize;
+	mach_port_t             sls_port = mach_host_self();
+	mach_msg_type_number_t  vmCount = HOST_VM_INFO_COUNT;
+	vm_statistics_data_t    vm;
+	kern_return_t           error;
+	
+	error = host_statistics(sls_port , HOST_VM_INFO , (host_info_t) &vm, &vmCount);
+	pagesize = (unsigned long)sysconf(_SC_PAGESIZE);
+	memsize = (vm.free_count + vm.inactive_count) * pagesize;//(vm.wire_count + vm.active_count + vm.inactive_count + vm.free_count + vm.zero_fill_count ) * pagesize;
+	ret = (size_t)(memsize/(1024*1024));
 #elif __linux__
 	{
 		struct sysinfo sysInf;
@@ -1204,7 +1148,7 @@ bool strhas(const char *cpTest, const char *cpPossible)
 }
 
 //A routine for loading numeric data from a text file
-unsigned int loadTextData(const char *cpFilename, vector<vector<float> > &dataVec,vector<string> &headerVec, const char delim)
+unsigned int loadTextData(const char *cpFilename, vector<vector<float> > &dataVec,vector<string> &headerVec, const char *delim)
 {
 	const unsigned int BUFFER_SIZE=4096;
 	char inBuffer[BUFFER_SIZE];
@@ -1218,7 +1162,7 @@ unsigned int loadTextData(const char *cpFilename, vector<vector<float> > &dataVe
 	if(!CFile)
 		return ERR_FILE_OPEN_FAIL;
 
-	//DroR the headers, if any
+	//Drop the headers, if any
 	string str;
 	vector<string> strVec;	
 	bool atHeader=true;
@@ -1254,9 +1198,20 @@ unsigned int loadTextData(const char *cpFilename, vector<vector<float> > &dataVe
 				float f;
 				//Assume we are not reading the header
 				atHeader=false;
+
+				vector<float> values;
 				//Confirm by checking all values
 				for(unsigned int ui=0; ui<num_fields; ui++)
 				{
+	
+					//stream_cast will fail in the case "1 2" if there
+					//is a tab delimiter specified. Check for only 02,3	
+					if(strVec[ui].find_first_not_of("0123456789.Ee+-") 
+										!= string::npos)
+					{
+						atHeader=true;
+						break;
+					}
 					//If any cast fails
 					//we are in the header
 					if(stream_cast(f,strVec[ui]))
@@ -1265,16 +1220,36 @@ unsigned int loadTextData(const char *cpFilename, vector<vector<float> > &dataVe
 						break;
 					}
 
+					values.push_back(f);
 				}
+
+				if(!atHeader)
+					break;
 			}
 		}
 
 	}
 
+	//Drop the blank bits from the field
 	stripZeroEntries(prevStrs);
+	//switch this out as being the header
 	if(prevStrs.size() == num_fields)
 		std::swap(headerVec,prevStrs);
 
+	if(atHeader)
+	{
+		//re-wind back to the beginning of the file
+		//as we didn't find a header/data split
+		CFile.clear(); // clear EOF bit
+		CFile.seekg(0,std::ios::beg);
+		CFile.getline(inBuffer,BUFFER_SIZE);
+		
+		
+		splitStrsRef(inBuffer,delim,strVec);	
+		stripZeroEntries(strVec);
+		num_fields=strVec.size();
+
+	}
 
 	float f;
 	std::stringstream ss;
@@ -1311,8 +1286,7 @@ unsigned int loadTextData(const char *cpFilename, vector<vector<float> > &dataVe
 		CFile.getline(inBuffer,BUFFER_SIZE);
 	}
 
-	if(atHeader)
-		return ERR_FILE_FORMAT_FAIL;
-
 	return 0;
 }
+
+
