@@ -16,25 +16,12 @@
  *	along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
-//OpenGL debugging macro
-#if DEBUG
-#define glError() { \
-		GLenum err = glGetError(); \
-		while (err != GL_NO_ERROR) { \
-					fprintf(stderr, "glError: %s caught at %s:%u\n", (char *)gluErrorString(err), __FILE__, __LINE__); \
-					err = glGetError(); \
-				} \
-		std::cerr << "glErr Clean " << __FILE__ << ":" << __LINE__ << std::endl; \
-}
-#else
-#define glError()
-#endif
-#include "quat.h"
-#include "common.h"
 #include "cameras.h"
-#include "XMLHelper.h"
+#include "mathfuncs.h"
+#include "xmlHelper.h"
+
+#include "commonConstants.h"
 
-#define GL_GLEXT_PROTOTYPES 1
 //MacOS is "special" and puts it elsewhere
 #ifdef __APPLE__
 	#include <OpenGL/glu.h>
@@ -53,20 +40,20 @@ using std::pair;
 //!Key types for property setting and getting via property grids
 enum
 {
-	KEY_PERSP_ORIGIN,
-	KEY_PERSP_VIEWDIR,
-	KEY_PERSP_UPDIR,
-	KEY_PERSP_FOV,
-	KEY_PERSPLOOKAT_ORIGIN,
-	KEY_PERSPLOOKAT_TARGET,
-	KEY_PERSPLOOKAT_UPDIRECTION,
-	KEY_PERSPLOOKAT_FOV,
+	KEY_LOOKAT_LOCK,
+	KEY_LOOKAT_ORIGIN,
+	KEY_LOOKAT_TARGET,
+	KEY_LOOKAT_UPDIRECTION,
+	KEY_LOOKAT_FOV,
+	KEY_LOOKAT_PROJECTIONMODE,
+	KEY_LOOKAT_ORTHOSCALE,
 };
 
 
 
 Camera::Camera() : origin(0.0f,0.0f,0.0f), viewDirection(0.0f,0.0f,-1.0f), upDirection(0.0f,0.0f,1.0f)
 {
+	lock=false;
 }
 
 Camera::~Camera()
@@ -91,28 +78,38 @@ Point3D Camera::getUpDirection() const
 
 void Camera::setOrigin(const Point3D &pt)
 {
+	if(lock)
+		return;
 	origin=pt;
 }
 
 void Camera::setViewDirection(const Point3D &pt)
 {
+	if(lock)
+		return;
 	viewDirection=pt;
 	viewDirection.normalise();
 }
 
 void Camera::setUpDirection(const Point3D &pt)
 {
+	if(lock)
+		return;
 	upDirection = pt;
 	upDirection.normalise();
 }
 
 void Camera::forwardsDolly(float moveRate)
 {
+	if(lock)
+		return;
 	origin=origin+ viewDirection*moveRate;
 }
 
 void Camera::move(float moveLR, float moveUD)
 {
+	if(lock)
+		return;
 	//Right is the cross product of up and 
 	//view direction (check sign)
 	//Up is simply the up vector
@@ -121,12 +118,16 @@ void Camera::move(float moveLR, float moveUD)
 
 void Camera::translate(float moveLR, float moveUD)
 {
+	if(lock)
+		return;
 	//This camera has no target. Just do plain move
 	move(moveLR,moveUD);
 }
 
 void Camera::pivot(float lrRad, float udRad)
 {
+	if(lock)
+		return;
 	Point3f viewNew, rotateAxis;
 
 	//rotate normalised rOrig around axis one then two
@@ -155,36 +156,66 @@ void Camera::pivot(float lrRad, float udRad)
 
 //=====
 
-CameraPerspective::CameraPerspective() : fovAngle(90.0f), nearPlane(1)
+
+//=====
+
+CameraLookAt::CameraLookAt() 
 {
-	typeNum=CAM_PERSP;
+	target=Point3D(0.0f,0.0f,0.0f);
+	origin=Point3D(0.0f,0.0f,1.0f);
+	viewDirection=Point3D(0.0f,0.0f,-1.0f);
+	upDirection=Point3D(0.0f,1.0f,0.0f);
+	typeNum=CAM_LOOKAT;
+	projectionMode=PROJECTION_MODE_PERSPECTIVE;
+	fovAngle=90.0f;
+	nearPlane=1.0f;
+	frustumDistortion=0.0f;
+
+	orthoScale=1.0f;
 }
 
-CameraPerspective::~CameraPerspective()
+Camera *CameraLookAt::clone() const
 {
-}
+	CameraLookAt *retCam = new CameraLookAt;
 
-Camera *CameraPerspective::clone() const
-{
-	CameraPerspective *retCam = new CameraPerspective;
 	retCam->origin =origin;
 	retCam->viewDirection=viewDirection;
 	retCam->upDirection =upDirection;
+	retCam->projectionMode=projectionMode;
+	retCam->orthoScale=orthoScale;
+	retCam->typeNum=typeNum;
 	retCam->userString=userString;
+	retCam->lock=lock;
 
+	retCam->target = target;
 	retCam->fovAngle = fovAngle;
 	retCam->nearPlane=nearPlane;
+	retCam->farPlane=farPlane;
+
 
 	return retCam;
 }
 
-void CameraPerspective::apply(float aspectRatio, const BoundCube &bc,bool loadIdentity) const
+CameraLookAt::~CameraLookAt()
 {
-	doPerspCalcs(aspectRatio,bc,loadIdentity);
 }
 
-void CameraPerspective::doPerspCalcs(float aspectRatio, const BoundCube &bc,bool loadIdentity) const
+void CameraLookAt::setTarget(const Point3D &pt)
 {
+	ASSERT(pt.sqrDist(origin)>10.0f*std::numeric_limits<float>::epsilon());
+	target=pt;
+	recomputeViewDirection();
+}
+
+Point3D CameraLookAt::getTarget() const
+{
+	return target;
+}
+
+void CameraLookAt::doPerspCalcs(float aspectRatio, const BoundCube &bc,bool loadIdentity) const
+{
+	ASSERT(projectionMode == PROJECTION_MODE_PERSPECTIVE);
+
 	glMatrixMode (GL_PROJECTION);
 	if(loadIdentity)
 		glLoadIdentity();
@@ -219,297 +250,44 @@ void CameraPerspective::doPerspCalcs(float aspectRatio, const BoundCube &bc,bool
 
 }
 
-void CameraPerspective::doPerspCalcs(float aspectRatio, const BoundCube &bc,bool loadIdentity, float left, float top) const
-{
-	ASSERT(false);
-}
-
-
-//Make a given bounding box visible, as much as possible
-void CameraPerspective::ensureVisible(const BoundCube &boundCube, unsigned int face)
-{
-	//FIXME: Implement me
-	// -- what does it mean to ensure visible for a free camera??
-	
-
-	ASSERT(false);
-
-}
-
-void CameraPerspective::recomputeUpDirection()
-{
-	//Use cross product of view and up to generate an across vector.
-	Point3D across;
-	upDirection.normalise();
-	across = viewDirection.crossProd(upDirection);
-	across.normalise();
-
-	//Regenerate up vector by reversing the cross with a normalised across vector 
-	upDirection = across.crossProd(viewDirection);
-
-	upDirection.normalise();
-}
-
-void CameraPerspective::getProperties(CameraProperties &p) const
-{
-	p.data.clear();
-	p.types.clear();
-	p.keys.clear();
-
-	vector<pair<string,string> > s;
-	vector<unsigned int> type,keys;
-
-	string ptStr;
-	stream_cast(ptStr,origin);
-	s.push_back(std::make_pair("Origin", ptStr));
-	type.push_back(PROPERTY_TYPE_POINT3D);
-	keys.push_back(KEY_PERSP_ORIGIN);
-	
-	stream_cast(ptStr,viewDirection);
-	s.push_back(std::make_pair("View Dir.", ptStr));
-	type.push_back(PROPERTY_TYPE_POINT3D);
-	keys.push_back(KEY_PERSP_VIEWDIR);
-	
-	stream_cast(ptStr,upDirection);
-	s.push_back(std::make_pair("Up Dir.", ptStr));
-	type.push_back(PROPERTY_TYPE_POINT3D);
-	keys.push_back(KEY_PERSP_UPDIR);
-
-
-	string tmp;
-	stream_cast(tmp,fovAngle);
-	s.push_back(std::make_pair("Field of View (deg)", tmp));
-	type.push_back(PROPERTY_TYPE_REAL);
-	keys.push_back(KEY_PERSP_FOV);
-
-	p.data.push_back(s);
-	p.keys.push_back(keys);
-	p.types.push_back(type);
-}
-
-bool CameraPerspective::setProperty(unsigned int key, const string &value)
-{
-	//TODO: IMPLEMENT ME
-	return false;
-}
-
-bool CameraPerspective::writeState(std::ostream &f, unsigned int format,
-	       					unsigned int nTabs) const
-{
-	using std::endl;
-	
-	f << tabs(nTabs) << "<perspective>" << endl;
-	ASSERT(userString.size());
-	f << tabs(nTabs+1) << "<userstring value=\"" << userString << "\"/>" << endl;
-			
-	f << tabs(nTabs+1) << "<origin x=\"" << origin[0] << "\" y=\"" << origin[1] <<
-	       	"\" z=\"" << origin[2] << "\"/>" << endl;
-	f << tabs(nTabs+1) << "<viewdirecton x=\"" << viewDirection[0] << "\" y=\"" << origin[1] <<
-	       	"\" z=\"" << viewDirection[2] << "\"/>" << endl;
-	f << tabs(nTabs+1) << "<updirection x=\"" << origin[0] << "\" y=\"" << origin[1] <<
-	       	"\" z=\"" << upDirection[2] << "\"/>" << endl;
-
-	f<< tabs(nTabs+1) <<  "<fovangle value=\"" << fovAngle << "\"/>" << endl;
-	f<< tabs(nTabs+1) <<  "<nearplane value=\"" << nearPlane << "\"/>" << endl;
-	f << tabs(nTabs) << "</perspective>" << endl;
-
-	return true;
-}
-
-bool CameraPerspective::readState(xmlNodePtr nodePtr)
-{
-	//Retrieve user string
-	if(XMLHelpFwdToElem(nodePtr,"userstring"))
-		return false;
-
-	xmlChar *xmlString=xmlGetProp(nodePtr,(const xmlChar *)"value");
-	if(!xmlString)
-		return false;
-	userString=(char *)xmlString;
-	xmlFree(xmlString);
-	if(!userString.size())
-		return false;
-
-	std::string tmpStr;	
-	//Retrieve origin
-	//====
-	if(XMLHelpFwdToElem(nodePtr,"origin"))
-		return false;
-	float x,y,z;
-	//--Get X value--
-	xmlString=xmlGetProp(nodePtr,(const xmlChar *)"x");
-	if(!xmlString)
-		return false;
-	tmpStr=(char *)xmlString;
-	xmlFree(xmlString);
-
-	//Check it is streamable
-	if(stream_cast(x,tmpStr))
-		return false;
-
-	//--Get Z value--
-	xmlString=xmlGetProp(nodePtr,(const xmlChar *)"y");
-	if(!xmlString)
-		return false;
-	tmpStr=(char *)xmlString;
-	xmlFree(xmlString);
-
-	//Check it is streamable
-	if(stream_cast(y,tmpStr))
-		return false;
-
-	//--Get Y value--
-	xmlString=xmlGetProp(nodePtr,(const xmlChar *)"z");
-	if(!xmlString)
-		return false;
-	tmpStr=(char *)xmlString;
-	xmlFree(xmlString);
-
-	//Check it is streamable
-	if(stream_cast(z,tmpStr))
-		return false;
-
-	origin=Point3D(x,y,z);
-	//====
-
-	
-	//Retrieve up direction
-	//====
-	if(XMLHelpFwdToElem(nodePtr,"updirection"))
-		return false;
-	//--Get X value--
-	xmlString=xmlGetProp(nodePtr,(const xmlChar *)"x");
-	if(!xmlString)
-		return false;
-	tmpStr=(char *)xmlString;
-	xmlFree(xmlString);
-
-	//Check it is streamable
-	if(stream_cast(x,tmpStr))
-		return false;
-
-	//--Get Z value--
-	xmlString=xmlGetProp(nodePtr,(const xmlChar *)"y");
-	if(!xmlString)
-		return false;
-	tmpStr=(char *)xmlString;
-	xmlFree(xmlString);
-
-	//Check it is streamable
-	if(stream_cast(y,tmpStr))
-		return false;
-
-	//--Get Y value--
-	xmlString=xmlGetProp(nodePtr,(const xmlChar *)"z");
-	if(!xmlString)
-		return false;
-	tmpStr=(char *)xmlString;
-	xmlFree(xmlString);
-
-	//Check it is streamable
-	if(stream_cast(z,tmpStr))
-		return false;
-	upDirection=Point3D(x,y,z);
-	//====
-	
-
-	//Get the FOV angle
-	//====
-	if(XMLHelpFwdToElem(nodePtr,"fovangle"))
-		return false;
-
-	xmlString=xmlGetProp(nodePtr,(const xmlChar *)"value");
-	if(!xmlString)
-		return false;
-	tmpStr=(char *)xmlString;
-
-	if(stream_cast(fovAngle,tmpStr))
-		return false;
-
-	xmlFree(xmlString);
-	
-	//====
-	
-	//Get the near plane
-	//====
-	if(XMLHelpFwdToElem(nodePtr,"nearplane"))
-		return false;
-
-	xmlString=xmlGetProp(nodePtr,(const xmlChar *)"value");
-	if(!xmlString)
-		return false;
-	tmpStr=(char *)xmlString;
-
-	if(stream_cast(nearPlane,tmpStr))
-		return false;
-
-	xmlFree(xmlString);
-	
-	//====
-	
-	return true;
-}	
-//=====
-
-CameraPerspLookAt::CameraPerspLookAt() 
-{
-	target=Point3D(0.0f,0.0f,0.0f);
-	origin=Point3D(0.0f,0.0f,1.0f);
-	viewDirection=Point3D(0.0f,0.0f,-1.0f);
-	upDirection=Point3D(0.0f,1.0f,0.0f);
-	typeNum=CAM_LOOKAT;
-}
-
-Camera *CameraPerspLookAt::clone() const
-{
-	CameraPerspLookAt *retCam = new CameraPerspLookAt;
-
-	retCam->origin =origin;
-	retCam->viewDirection=viewDirection;
-	retCam->upDirection =upDirection;
-	retCam->userString=userString;
-
-	retCam->fovAngle = fovAngle;
-	retCam->nearPlane=nearPlane;
-
-	retCam->target = target;
-
-	return retCam;
-}
-
-CameraPerspLookAt::~CameraPerspLookAt()
-{
-}
-
-void CameraPerspLookAt::setTarget(const Point3D &pt)
-{
-	ASSERT(pt.sqrDist(origin)>10.0f*std::numeric_limits<float>::epsilon());
-	target=pt;
-	recomputeViewDirection();
-}
-
-Point3D CameraPerspLookAt::getTarget() const
-{
-	return target;
-}
-
-
-void CameraPerspLookAt::setOrigin(const Point3D &newOrigin)
+void CameraLookAt::setOrigin(const Point3D &newOrigin)
 {
+	if(lock)
+		return;
 	ASSERT(newOrigin.sqrDist(target)>std::numeric_limits<float>::epsilon());
 	origin=newOrigin;
 	recomputeViewDirection();
 }
 
-void CameraPerspLookAt::apply(float aspect, const BoundCube &bc, bool loadIdentity) const
+void CameraLookAt::apply(float aspect, const BoundCube &bc, bool loadIdentity) const
 {
+
 	glMatrixMode (GL_PROJECTION);
 	if(loadIdentity)
 		glLoadIdentity();
-
+	
 	farPlane = 1.5*bc.getMaxDistanceToBox(origin);
-	gluPerspective(fovAngle/2.0,aspect,nearPlane,farPlane);
-	glMatrixMode(GL_MODELVIEW);
+	switch(projectionMode)
+	{
+
+		case PROJECTION_MODE_PERSPECTIVE:
+		{
+
+			gluPerspective(fovAngle/2.0,aspect,nearPlane,farPlane);
+			glMatrixMode(GL_MODELVIEW);
+			break;
+
+		}
+		case PROJECTION_MODE_ORTHOGONAL:
+		{
+			glOrtho(-orthoScale*aspect,orthoScale*aspect,-orthoScale,orthoScale,nearPlane,farPlane);
+			glMatrixMode(GL_MODELVIEW);
+			break;
+		}
+		default:
+			ASSERT(false);
+
+	}
 
 	ASSERT(origin.sqrDist(target)>std::numeric_limits<float>::epsilon());
 	gluLookAt(origin[0],origin[1],origin[2],
@@ -517,7 +295,7 @@ void CameraPerspLookAt::apply(float aspect, const BoundCube &bc, bool loadIdenti
 				upDirection[0],upDirection[1],upDirection[2]);
 }
 
-void CameraPerspLookAt::apply(float aspect,const BoundCube &b,bool loadIdentity,
+void CameraLookAt::apply(float aspect,const BoundCube &b,bool loadIdentity,
 						float leftRestrict,float rightRestrict, 
 						float bottomRestrict,float topRestrict) const
 {
@@ -528,57 +306,134 @@ void CameraPerspLookAt::apply(float aspect,const BoundCube &b,bool loadIdentity,
 	if(loadIdentity)
 		glLoadIdentity();
 
-	float width,height;
-	height = tan(fovAngle/2.0*M_PI/180)*nearPlane;
-	width= height*aspect;
-	
 	farPlane = 1.5*b.getMaxDistanceToBox(origin);
+	switch(projectionMode)
+	{
 
-	//Frustum uses eye coordinates.	
-	glFrustum(leftRestrict*width,rightRestrict*width,bottomRestrict*height,
-					topRestrict*height,nearPlane,farPlane);
+		case PROJECTION_MODE_PERSPECTIVE:
+		{
+			float width,height;
+			height = tan(fovAngle/2.0*M_PI/180)*nearPlane;
+			width= height*aspect;
+			
+
+			//Frustum uses eye coordinates.	
+			if(fabs(frustumDistortion) < std::numeric_limits<float>::epsilon())
+				glFrustum(leftRestrict*width,rightRestrict*width,bottomRestrict*height,
+								topRestrict*height,nearPlane,farPlane);
+			else
+			{
+				float workingDist=farPlane;//sqrtf((target-origin).sqrMag());
+				glFrustum(leftRestrict*width+frustumDistortion*nearPlane/workingDist,
+						rightRestrict*width+frustumDistortion*nearPlane/workingDist,
+						bottomRestrict*height,	topRestrict*height,nearPlane,farPlane);
+			}
+			break;
+		}
+		case PROJECTION_MODE_ORTHOGONAL:
+		{
+			float l,r,b,t;
+
+			//FIXME:I have no idea why it is 2x...AFAIK it should just be ONE.
+			//but this works, and one does not.
+			l = 2*leftRestrict*orthoScale*aspect;
+			r= 2*rightRestrict*orthoScale*aspect;
+			b= 2*bottomRestrict*orthoScale;
+			t = 2*topRestrict*orthoScale;
+
+			glOrtho(l,r,b,t,nearPlane,farPlane);
+			break;
+		}
+		default:
+			ASSERT(false);
+	}
+	
 	glMatrixMode(GL_MODELVIEW);
 
 	ASSERT(origin.sqrDist(target)>std::numeric_limits<float>::epsilon());
 	gluLookAt(origin[0],origin[1],origin[2],
 				target[0],target[1],target[2],
 				upDirection[0],upDirection[1],upDirection[2]);
+	
 }
 
-void CameraPerspLookAt::translate(float moveLR, float moveUD)
+void CameraLookAt::translate(float moveLR, float moveUD)
 {
-	moveLR=moveLR*sqrt(target.sqrDist(origin));
-	moveUD=moveUD*sqrt(target.sqrDist(origin));
+	if(lock)
+		return;
+	float fovMultiplier=1.0f;
+	if(projectionMode== PROJECTION_MODE_PERSPECTIVE)
+	{
+
+		//Try to move such that the target sweeps our field of view
+		//at a constant rate. Standard normaliser is view length at
+		//a 90* camera
+		//Use tan.. to normalise motion rate 
+		//Prevent numerical error near tan( 90*)
+		if(fovAngle < 175.0f)
+			fovMultiplier = tan(fovAngle/2.0*M_PI/180.0);
+		else
+			fovMultiplier = tan(175.0f/2.0*M_PI/180.0);
+	}
+	
+
+	moveLR=moveLR*sqrtf(target.sqrDist(origin)*fovMultiplier);
+	moveUD=moveUD*sqrtf(target.sqrDist(origin)*fovMultiplier);
 
 	origin+=upDirection*moveUD + (upDirection.crossProd(viewDirection))*moveLR;
 	target+=upDirection*moveUD + (upDirection.crossProd(viewDirection))*moveLR;
 }
 
-void CameraPerspLookAt::forwardsDolly(float moveRate)
+void CameraLookAt::forwardsDolly(float moveRate)
 {
+	if(lock)
+		return;
 
-	//Prevent camera orientation inversion, which occurs when moving past the target
-	if((viewDirection*moveRate).sqrMag() > target.sqrDist(origin))
+	if(projectionMode == PROJECTION_MODE_PERSPECTIVE)
 	{
-		if((target-origin).sqrMag() < sqrt(std::numeric_limits<float>::epsilon()))
-				return;
+		Point3D newOrigin;
 
-		//Yes, this simplifies analytically. However i think the numerics come into play.
-		float moveInv = 1.0/(fabs(moveRate) + sqrt(std::numeric_limits<float>::epsilon()));
-		origin=origin+viewDirection*moveInv/(1.0+moveInv);
+		//Prevent camera orientation inversion, which occurs when moving past the target
+		if(moveRate > sqrt(target.sqrDist(origin)))
+		{
+			if((target-origin).sqrMag() < sqrtf(std::numeric_limits<float>::epsilon()))
+					return;
+
+			//Yes, this simplifies analytically. However i think the numerics come into play.
+			float moveInv = 1.0/(fabs(moveRate) + std::numeric_limits<float>::epsilon());
+			newOrigin=origin+viewDirection*moveInv/(1.0+moveInv);
+
+		}
+		else
+		{
+			//scale moverate by orbit distance
+			moveRate = moveRate*sqrtf(target.sqrDist(origin));
+			newOrigin=origin+viewDirection*moveRate;
+		}
+
+		//Only accept origin change if it is sufficiently far from the target
+		if(newOrigin.sqrDist(target)>sqrtf(std::numeric_limits<float>::epsilon()))
+			origin=newOrigin;
 	}
 	else
 	{
-		//scale moverate by orbit distance
-		moveRate = 0.05*moveRate*sqrt(target.sqrDist(origin));
-		origin=origin+viewDirection*moveRate;
+		float deltaSqr;
+		deltaSqr = (target-origin).sqrMag();
+		if(deltaSqr< sqrtf(std::numeric_limits<float>::epsilon()))
+			return;
+
+		Point3D virtualOrigin;
+		virtualOrigin = origin+viewDirection*moveRate;
+		orthoScale*=virtualOrigin.sqrDist(target)/deltaSqr;
 	}
 }
 
 
 //Clockwise roll looking from camera view by rollRad radians
-void CameraPerspLookAt::roll(float rollRad)
+void CameraLookAt::roll(float rollRad)
 {
+	if(lock)
+		return;
 	Point3f rNew,rotateAxis;
 
 	rotateAxis.fx=viewDirection[0];
@@ -594,8 +449,10 @@ void CameraPerspLookAt::roll(float rollRad)
 	recomputeUpDirection();
 }
 
-void CameraPerspLookAt::pivot(float leftRightRad,float updownRad)
+void CameraLookAt::pivot(float leftRightRad,float updownRad)
 {
+	if(lock)
+		return;
 
 	Point3f rNew,rotateAxis;
 	Point3D tmp;
@@ -625,15 +482,17 @@ void CameraPerspLookAt::pivot(float leftRightRad,float updownRad)
 	newDir+= Point3D(rNew.fx,rNew.fy,rNew.fz);
 	target = target+newDir;
 	target.normalise(); 
-	target*=sqrt((target).sqrDist(origin));
+	target*=sqrtf((target).sqrDist(origin));
 
 	recomputeViewDirection();
 	recomputeUpDirection();
 }
 
 //Make a given bounding box visible, as much as possible
-void CameraPerspLookAt::ensureVisible(const BoundCube &boundCube, unsigned int face)
+void CameraLookAt::ensureVisible(const BoundCube &boundCube, unsigned int face)
 {
+	if(lock)
+		return;
 	//Face is defined by the following net
 	//	1
 	//  2   3   4
@@ -701,6 +560,8 @@ void CameraPerspLookAt::ensureVisible(const BoundCube &boundCube, unsigned int f
 			faceSize[0]=boundCube.getSize(1);
 			faceSize[1]=boundCube.getSize(2);
 			break;
+		default:
+			ASSERT(false);
 	}	
 
 
@@ -716,10 +577,21 @@ void CameraPerspLookAt::ensureVisible(const BoundCube &boundCube, unsigned int f
 	target=boxCentroid;
 
 	float outDistance;
-	outDistance=boxToFrontDist+halfMaxFaceDim/tan((fovAngle*M_PI/180)/2.0f);
-	//Minimal camera distance is given trigonometrically. Add aditional 1 to ensure that nearplane does not clip object
-	//then multiply by 1.4 to give a bit of border.
-	origin=boxCentroid+faceOutVector*1.4*(1.0+outDistance);
+	if(projectionMode == PROJECTION_MODE_PERSPECTIVE)
+	{
+		//Minimal camera distance is given trigonometrically. 
+		//Add aditional 1 to ensure that nearplane does not clip object
+		outDistance=1.0+boxToFrontDist+halfMaxFaceDim/tan((fovAngle*M_PI/180)/2.0f);
+	}
+	else
+	{
+		outDistance=boxToFrontDist+halfMaxFaceDim;
+	}
+
+	//Multiply by 1.4 to give a bit of border.
+	origin=boxCentroid+faceOutVector*1.4*outDistance;
+
+	orthoScale = sqrtf(target.sqrDist(origin))/2.0;
 
 
 	//Set the default up direction
@@ -732,15 +604,30 @@ void CameraPerspLookAt::ensureVisible(const BoundCube &boundCube, unsigned int f
 	nearPlane = 1;
 }
 
-void CameraPerspLookAt::recomputeViewDirection()
+void CameraLookAt::recomputeViewDirection()
 {
 	viewDirection=origin-target;
 	viewDirection.normalise();
 }
-
-
-void CameraPerspLookAt::move(float moveLRAngle, float moveUDAngle)
+void CameraLookAt::recomputeUpDirection()
 {
+	//Use cross product of view and up to generate an across vector.
+	Point3D across;
+	upDirection.normalise();
+	across = viewDirection.crossProd(upDirection);
+	across.normalise();
+
+	//Regenerate up vector by reversing the cross with a normalised across vector 
+	upDirection = across.crossProd(viewDirection);
+
+	upDirection.normalise();
+}
+
+
+void CameraLookAt::move(float moveLRAngle, float moveUDAngle)
+{
+	if(lock)
+		return;
 
 	//Think of the camera as moving around the surface of a sphere
 	Point3f curOrig;
@@ -779,7 +666,7 @@ void CameraPerspLookAt::move(float moveLRAngle, float moveUDAngle)
 	recomputeViewDirection();
 }
 
-void CameraPerspLookAt::getProperties(CameraProperties &p) const
+void CameraLookAt::getProperties(CameraProperties &p) const
 {
 	p.data.clear();
 	p.types.clear();
@@ -788,70 +675,118 @@ void CameraPerspLookAt::getProperties(CameraProperties &p) const
 	vector<pair<string,string> > s;
 	vector<unsigned int> type,keys;
 
+	if(lock)
+		s.push_back(std::make_pair("Lock","1"));
+	else
+		s.push_back(std::make_pair("Lock","0"));
+
+	type.push_back(PROPERTY_TYPE_BOOL);
+	keys.push_back(KEY_LOOKAT_LOCK);
+
 	string ptStr;
 	stream_cast(ptStr,origin);
 	s.push_back(std::make_pair("Origin", ptStr));
 	type.push_back(PROPERTY_TYPE_POINT3D);
-	keys.push_back(KEY_PERSPLOOKAT_ORIGIN);
+	keys.push_back(KEY_LOOKAT_ORIGIN);
 	
 	stream_cast(ptStr,target);
 	s.push_back(std::make_pair("Target", ptStr));
 	type.push_back(PROPERTY_TYPE_POINT3D);
-	keys.push_back(KEY_PERSPLOOKAT_TARGET);
+	keys.push_back(KEY_LOOKAT_TARGET);
 	
 	stream_cast(ptStr,upDirection);
 	s.push_back(std::make_pair("Up Dir.", ptStr));
 	type.push_back(PROPERTY_TYPE_POINT3D);
-	keys.push_back(KEY_PERSPLOOKAT_UPDIRECTION);
+	keys.push_back(KEY_LOOKAT_UPDIRECTION);
 
+	vector<pair<unsigned int,string> > choices;
 	string tmp;
-	stream_cast(tmp,fovAngle);
-	s.push_back(std::make_pair("Field of View (deg)", tmp));
-	type.push_back(PROPERTY_TYPE_REAL);
-	keys.push_back(KEY_PERSPLOOKAT_FOV);
+	
 
+	tmp="Perspective";
+	choices.push_back(make_pair((unsigned int)PROJECTION_MODE_PERSPECTIVE,tmp));
+	tmp="Orthogonal";
+	choices.push_back(make_pair((unsigned int)PROJECTION_MODE_ORTHOGONAL,tmp));
+	tmp= choiceString(choices,projectionMode);
+	
+	s.push_back(std::make_pair("Projection", tmp));
+	type.push_back(PROPERTY_TYPE_CHOICE);
+	keys.push_back(KEY_LOOKAT_PROJECTIONMODE);
+
+	switch(projectionMode)
+	{
+		case PROJECTION_MODE_PERSPECTIVE:
+			stream_cast(tmp,fovAngle);
+			s.push_back(std::make_pair("Field of View (deg)", tmp));
+			type.push_back(PROPERTY_TYPE_REAL);
+			keys.push_back(KEY_LOOKAT_FOV);
+			break;
+		case PROJECTION_MODE_ORTHOGONAL:
+			stream_cast(tmp,orthoScale);
+			s.push_back(std::make_pair("View size", tmp));
+			type.push_back(PROPERTY_TYPE_REAL);
+			keys.push_back(KEY_LOOKAT_ORTHOSCALE);
+			break;
+
+	}
 
 	p.data.push_back(s);
 	p.keys.push_back(keys);
 	p.types.push_back(type);
 }
 
-bool CameraPerspLookAt::setProperty(unsigned int key, const string &value)
+bool CameraLookAt::setProperty(unsigned int key, const string &value)
 {
 
 	switch(key)
 	{
-		case KEY_PERSPLOOKAT_ORIGIN:
+		case KEY_LOOKAT_LOCK:
+		{
+			if(value == "1")
+				lock=true;
+			else if (value == "0")
+				lock=false;
+			else
+				return false;
+
+			break;
+		}
+		case KEY_LOOKAT_ORIGIN:
 		{
 			Point3D newPt;
-			if(!newPt.parse(value))
+			if(!parsePointStr(value,newPt))
 				return false;
 
 			//Disallow origin to be set to same as target
-			if(newPt.sqrDist(target) < sqrt(std::numeric_limits<float>::epsilon()))
+			if(newPt.sqrDist(target) < sqrtf(std::numeric_limits<float>::epsilon()))
 				return false;
 
 			origin= newPt;
 
 			break;
 		}
-		case KEY_PERSPLOOKAT_TARGET:
+		case KEY_LOOKAT_TARGET:
 		{
 			Point3D newPt;
-			if(!newPt.parse(value))
+			if(!parsePointStr(value,newPt))
 				return false;
 
 			//Disallow origin to be set to same as target
-			if(newPt.sqrDist(origin) < sqrt(std::numeric_limits<float>::epsilon()))
+			if(newPt.sqrDist(origin) < sqrtf(std::numeric_limits<float>::epsilon()))
 				return false;
 			target = newPt;
 
 			break;
 		}
-		case KEY_PERSPLOOKAT_UPDIRECTION:
+		case KEY_LOOKAT_UPDIRECTION:
 		{
 			Point3D newDir;
-			if(!newDir.parse(value))
+			if(!parsePointStr(value,newDir))
+				return false;
+
+			//View direction and up direction may not be the same
+			if(viewDirection.crossProd(newDir).sqrMag() < 
+				sqrtf(std::numeric_limits<float>::epsilon()))
 				return false;
 
 			upDirection=newDir;
@@ -860,7 +795,7 @@ bool CameraPerspLookAt::setProperty(unsigned int key, const string &value)
 			recomputeUpDirection();
 			break;
 		}
-		case KEY_PERSPLOOKAT_FOV:
+		case KEY_LOOKAT_FOV:
 		{
 			float newFOV;
 			if(stream_cast(newFOV,value))
@@ -869,13 +804,55 @@ bool CameraPerspLookAt::setProperty(unsigned int key, const string &value)
 			fovAngle=newFOV;
 			break;
 		}
+		case KEY_LOOKAT_PROJECTIONMODE:
+		{
+			std::string tmp;
+			size_t ltmp;
+			if(value == "Perspective")
+				ltmp=PROJECTION_MODE_PERSPECTIVE;
+			else if( value == "Orthogonal")
+			{
+				if(projectionMode!=PROJECTION_MODE_ORTHOGONAL)
+				{
+					//use the distance to the target as the orthographic 
+					//scaling size (size of parallel frustrum)
+					orthoScale=sqrtf(target.sqrDist(origin));
+				}
+
+				ltmp=PROJECTION_MODE_ORTHOGONAL;
+
+			}
+			else
+			{
+				ASSERT(false);
+				return false;
+			}
+			
+			if(ltmp>=PROJECTION_MODE_ENUM_END)
+				return false;
+			
+			projectionMode=ltmp;
+			
+			break;
+		}
+		case KEY_LOOKAT_ORTHOSCALE:
+		{
+			float newOrthoScale;
+			if(stream_cast(newOrthoScale,value))
+				return false;
+
+			orthoScale=newOrthoScale;
+			break;
+		}
+
+
 		default:
 			ASSERT(false);
 	}
 	return true;
 }
 
-bool CameraPerspLookAt::writeState(std::ostream &f, unsigned int format,
+bool CameraLookAt::writeState(std::ostream &f, unsigned int format,
 	       					unsigned int nTabs) const
 {
 	using std::endl;
@@ -883,7 +860,13 @@ bool CameraPerspLookAt::writeState(std::ostream &f, unsigned int format,
 	f << tabs(nTabs) << "<persplookat>" << endl;
 	ASSERT(userString.size());
 	f << tabs(nTabs+1) << "<userstring value=\"" << userString << "\"/>" << endl;
-			
+	f << tabs(nTabs+1) << "<projectionmode value=\""<< projectionMode << "\"/>" << endl;
+	f << tabs(nTabs+1) << "<orthoscale value=\""<< orthoScale << "\"/>" << endl;
+	
+	if(lock)	
+		f<< tabs(nTabs+1) <<  "<lock value=\"1\"/>" << endl;
+	else
+		f<< tabs(nTabs+1) <<  "<lock value=\"0\"/>" << endl;
 	f << tabs(nTabs+1) << "<origin x=\"" << origin[0] << "\" y=\"" << origin[1] <<
 	       	"\" z=\"" << origin[2] << "\"/>" << endl;
 	f << tabs(nTabs+1) << "<target x=\"" << target[0] << "\" y=\"" << target[1] <<
@@ -898,26 +881,51 @@ bool CameraPerspLookAt::writeState(std::ostream &f, unsigned int format,
 	return true;
 }
 		
-bool CameraPerspLookAt::readState(xmlNodePtr nodePtr)
+bool CameraLookAt::readState(xmlNodePtr nodePtr)
 {
 	//Retrieve user string
-	if(XMLHelpFwdToElem(nodePtr,"userstring"))
+	if(!XMLGetNextElemAttrib(nodePtr,userString,"userstring","value"))
 		return false;
-
-	xmlChar *xmlString=xmlGetProp(nodePtr,(const xmlChar *)"value");
-	if(!xmlString)
-		return false;
-	userString=(char *)xmlString;
-	xmlFree(xmlString);
 	if(!userString.size())
 		return false;
 
-	std::string tmpStr;	
+	std::string tmpStr;
+
+	//Retrieve projection mode
+	if(!XMLGetNextElemAttrib(nodePtr,projectionMode,"projectionmode","value"))
+		return false;
+	if(projectionMode > PROJECTION_MODE_ENUM_END)
+		return false;
+
+	//Retrieve orthographic scaling
+	if(!XMLGetNextElemAttrib(nodePtr,orthoScale,"orthoscale","value"))
+		return false;
+	if(orthoScale <=0 || std::isnan(orthoScale))
+		return false;
+
+	
+	float x,y,z;
+	xmlChar *xmlString;
+
+
+	//retrieve lock state
+	if(XMLHelpFwdToElem(nodePtr,"lock"))
+		return false;
+	xmlString=xmlGetProp(nodePtr,(const xmlChar *)"value");
+
+	std::string strTmp;
+	strTmp=(char*)xmlString;
+	if(strTmp == "1")
+		lock=true;
+	else if(strTmp == "0")
+		lock=false;
+	else
+		return false;
+
 	//Retrieve origin
 	//====
 	if(XMLHelpFwdToElem(nodePtr,"origin"))
 		return false;
-	float x,y,z;
 	//--Get X value--
 	xmlString=xmlGetProp(nodePtr,(const xmlChar *)"x");
 	if(!xmlString)
@@ -1035,42 +1043,32 @@ bool CameraPerspLookAt::readState(xmlNodePtr nodePtr)
 
 	//Get the FOV angle
 	//====
-	if(XMLHelpFwdToElem(nodePtr,"fovangle"))
+	if(!XMLGetNextElemAttrib(nodePtr,fovAngle,"fovangle","value"))
 		return false;
-
-	xmlString=xmlGetProp(nodePtr,(const xmlChar *)"value");
-	if(!xmlString)
+	if(fovAngle<=0)
 		return false;
-	tmpStr=(char *)xmlString;
-
-	if(stream_cast(fovAngle,tmpStr))
-		return false;
-
-	xmlFree(xmlString);
-	
 	//====
 	
 	//Get the near plane
 	//====
-	if(XMLHelpFwdToElem(nodePtr,"nearplane"))
+	if(!XMLGetNextElemAttrib(nodePtr,nearPlane,"nearplane","value"))
 		return false;
-
-	xmlString=xmlGetProp(nodePtr,(const xmlChar *)"value");
-	if(!xmlString)
-		return false;
-	tmpStr=(char *)xmlString;
-
-	if(stream_cast(nearPlane,tmpStr))
-		return false;
-
-	xmlFree(xmlString);
-	
 	//====
 
 	recomputeViewDirection();	
 	return true;
 }	
 
+float CameraLookAt::getViewWidth(float depth) const
+{
+	if(projectionMode == PROJECTION_MODE_PERSPECTIVE)
+		return depth*tan(fovAngle/2.0f*M_PI/180.0);
+	else if(projectionMode == PROJECTION_MODE_ORTHOGONAL)
+		return -orthoScale*2.0f; //FIXME: Why is this negative??!
+
+	ASSERT(false);
+}
+
 std::ostream& operator<<(std::ostream &strm, const Camera &c)
 {
 	strm << "origin: " << c.origin << std::endl;
@@ -1079,18 +1077,7 @@ std::ostream& operator<<(std::ostream &strm, const Camera &c)
 	return strm;
 }
 
-std::ostream& operator<<(std::ostream &strm, const CameraPerspective &c)
-{
-	strm << "origin: " << c.origin << std::endl;
-	strm << "View Direction: " << c.viewDirection << std::endl;	
-	strm << "Up Direction: "<< c.upDirection << std::endl;
-
-	strm << "FOV (deg) : " << c.fovAngle << std::endl;
-	strm << "Clip plane: " << c.nearPlane << " (near) " << std::endl;
-	return strm;
-}
-
-std::ostream& operator<<(std::ostream &strm, const CameraPerspLookAt &c)
+std::ostream& operator<<(std::ostream &strm, const CameraLookAt &c)
 {
 	strm << "origin: " << c.origin << std::endl;
 	strm << "Target : " << c.target << std::endl;
@@ -1105,3 +1092,4 @@ std::ostream& operator<<(std::ostream &strm, const CameraPerspLookAt &c)
 }
 
 
+
