@@ -28,11 +28,12 @@
 
 #include "drawables.h"
 
+#include "colourmap.h"
+
+#include "isoSurface.h"
 
 #include <limits>
-#include "quat.h"
-
-//DEBUG ONLY
+#include "mathfuncs.h"
 
 //OpenGL debugging macro
 #if DEBUG
@@ -53,6 +54,7 @@
 //Static class variables
 //====
 const Camera *DrawGLText::curCamera = 0;
+const Camera *DrawField3D::curCam = 0;
 //==
 
 
@@ -81,18 +83,6 @@ DrawPoint::~DrawPoint()
 {
 }
 
-DrawableObj* DrawPoint::clone() const
-{
-	DrawPoint *d = new DrawPoint;
-
-	d->origin=origin;
-	d->r=r;
-	d->g=g;
-	d->b=b;
-	d->a=a;
-
-	return d;
-}
 
 
 
@@ -129,22 +119,6 @@ DrawVector::~DrawVector()
 {
 }
 
-DrawableObj* DrawVector::clone() const
-{
-	DrawVector *d = new DrawVector;
-
-	d->origin=origin;
-	d->vector=vector;
-	d->r=r;
-	d->g=g;
-	d->b=b;
-	d->a=a;
-
-	d->arrowSize=arrowSize;
-	d->scaleArrow=scaleArrow;
-
-	return d;
-}
 
 void DrawVector::getBoundingBox(BoundCube &b) const 
 {
@@ -179,6 +153,59 @@ void DrawVector::draw() const
 	glEnd();
 }
 
+void DrawVector::recomputeParams(const std::vector<Point3D> &vecs, 
+			const std::vector<float> &scalars, unsigned int mode)
+{
+	switch(mode)
+	{
+		case DRAW_VECTOR_BIND_ORIENTATION:
+			ASSERT(vecs.size() ==1 && scalars.size() ==0);
+			vector=vecs[0];
+			break;
+		case DRAW_VECTOR_BIND_ORIGIN:
+			ASSERT(scalars.size() == 1 && vecs.size()==0);
+			origin=vecs[0];
+			break;
+		default:
+			ASSERT(false);
+	}
+}
+DrawTriangle::DrawTriangle() : r(1.0f), g(1.0f),b(1.0f),a(1.0f)
+{
+}
+
+DrawTriangle::~DrawTriangle()
+{
+}
+
+void DrawTriangle::setVertex(unsigned int ui, const Point3D &pt)
+{
+	ASSERT(ui < 3);
+	vertices[ui] = pt;
+}
+
+void DrawTriangle::setColour(float rnew, float gnew, float bnew, float anew)
+{
+	r=rnew;
+	g=gnew;
+	b=bnew;
+	a=anew;
+}
+
+void DrawTriangle::draw() const
+{
+	glColor4f(r,g,b,a);
+	glBegin(GL_TRIANGLES);
+		glVertex3f((vertices[0])[0],
+			(vertices[0])[1], (vertices[0])[2]);
+		glVertex3f((vertices[1])[0],
+			(vertices[1])[1], (vertices[1])[2]);
+		glVertex3f((vertices[2])[0],
+			(vertices[2])[1], (vertices[2])[2]);
+	glEnd();
+}
+
+
 DrawSphere::DrawSphere() : radius(1.0f), latSegments(8),longSegments(8)
 {
 	q=gluNewQuadric();
@@ -190,17 +217,6 @@ DrawSphere::~DrawSphere()
 }
 
 
-DrawableObj* DrawSphere::clone() const
-{
-	DrawSphere *d=new DrawSphere;
-
-	d->origin=origin;
-	d->radius=radius;
-	d->latSegments=latSegments;
-	d->longSegments=longSegments;
-
-	return d;
-}
 
 void DrawSphere::getBoundingBox(BoundCube &b) const
 {
@@ -248,6 +264,24 @@ void DrawSphere::draw() const
 	glPopMatrix();
 }
 
+
+void DrawSphere::recomputeParams(const vector<Point3D> &vecs, 
+			const vector<float> &scalars, unsigned int mode)
+{
+	switch(mode)
+	{
+		case DRAW_SPHERE_BIND_ORIGIN:
+			ASSERT(vecs.size() ==1 && scalars.size() ==0);
+			origin=vecs[0];
+			break;
+		case DRAW_SPHERE_BIND_RADIUS:
+			ASSERT(scalars.size() == 1 && vecs.size()==0);
+			radius=scalars[0];
+			break;
+		default:
+			ASSERT(false);
+	}
+}
 //===========
 
 DrawCylinder::DrawCylinder() : radius(1.0f), 
@@ -268,44 +302,18 @@ DrawCylinder::~DrawCylinder()
 	gluDeleteQuadric(qCap[1]);
 }
 
-DrawableObj* DrawCylinder::clone() const
-{
-	DrawCylinder *d=new DrawCylinder;
-
-	d->origin=origin;
-	d->direction=direction;
-	
-	d->slices=slices;
-	d->stacks=stacks;
-
-	d->radius=radius;
-	d->radius=radius;
-
-	d->radiiLocked=true;
-
-	return d;
-}
 
 void DrawCylinder::setOrigin(const Point3D& pt)
 {
 	origin=pt;
 }
 
-Point3D *DrawCylinder::originPtr()
-{
-	return &origin;
-}
-
 
 void DrawCylinder::setDirection(const Point3D &p)
 {
 	direction=p;
 }
 
-Point3D *DrawCylinder::directionPtr()
-{
-	return &direction;
-}
 
 void DrawCylinder::draw() const
 {
@@ -321,7 +329,7 @@ void DrawCylinder::draw() const
 
 	float angle = dir.angle(dirNormal);
 	dir = dir.crossProd(dirNormal);
-	float length=sqrt(direction.sqrMag());
+	float length=sqrtf(direction.sqrMag());
 
 	glRotatef(angle*180.0f/M_PI,dir[0],dir[1],dir[2]);
 
@@ -341,12 +349,7 @@ void DrawCylinder::draw() const
 	}
 	else
 	{
-		gluDisk(qCap[0],0,radius,slices,1);
-		gluCylinder(q,radius,radius, length,slices,stacks);
-
-		//Draw the end cap at z=l	
-		glTranslatef(0,0,length);
-		gluDisk(qCap[1],0,radius,slices,1);
+		ASSERT(false);
 	}
 
 	glPopMatrix();
@@ -367,13 +370,30 @@ void DrawCylinder::setRadius(float rad)
 	radius=rad;
 }
 
-float *DrawCylinder::radiusPtr()
+void DrawCylinder::recomputeParams(const vector<Point3D> &vecs, 
+			const vector<float> &scalars, unsigned int mode)
 {
-	ASSERT(radiiLocked);
-	
-	return &radius;
+	switch(mode)
+	{
+		case DRAW_CYLINDER_BIND_ORIGIN:
+			ASSERT(vecs.size() ==1 && scalars.size() ==0);
+			origin=vecs[0];
+			break;
+
+		case DRAW_CYLINDER_BIND_DIRECTION:
+			ASSERT(vecs.size() ==1 && scalars.size() ==0);
+			direction=vecs[0];
+			break;
+		case DRAW_CYLINDER_BIND_RADIUS:
+			ASSERT(scalars.size() == 1 && vecs.size()==0);
+			radius=scalars[0];
+			break;
+		default:
+			ASSERT(false);
+	}
 }
 
+
 void DrawCylinder::setLength(float len)
 {
 	direction=direction.normalise()*len;
@@ -437,19 +457,24 @@ DrawManyPoints::DrawManyPoints() : r(1.0f),g(1.0f),b(1.0f),a(1.0f), size(1.0f)
 DrawManyPoints::~DrawManyPoints() 
 {
 	wantsLight=false;
+	haveCachedBounds=false;
 }
 
-DrawableObj* DrawManyPoints::clone() const
+void DrawManyPoints::getBoundingBox(BoundCube &b) const
 {
-	DrawManyPoints *d=new DrawManyPoints;
 
-	d->pts.resize(pts.size());
-	std::copy(pts.begin(),pts.end(),d->pts.begin());
+	//Update the cache as needed
+	if(!haveCachedBounds)
+	{
+		haveCachedBounds=true;
+		cachedBounds.setBounds(pts);
+	}
 
-	d->wantsLight=false;
-	return d;
+	b=cachedBounds;
+	return;
 }
 
+
 void DrawManyPoints::clear()
 {
 	pts.clear();
@@ -461,14 +486,15 @@ void DrawManyPoints::addPoints(const vector<Point3D> &vp)
 	std::copy(vp.begin(),vp.end(),pts.begin());
 }
 
-
+/*
 void DrawManyPoints::addPoints(const vector<IonHit> &vp)
 {
 	pts.reserve(pts.size()+vp.size());
 	for(size_t ui=0; ui<vp.size(); ui++)
 		pts.push_back(vp[ui].getPos());
+	haveCachedBounds=false;
 }
-
+*/
 void DrawManyPoints::shuffle()
 {
 	std::random_shuffle(pts.begin(),pts.end());
@@ -478,6 +504,7 @@ void DrawManyPoints::shuffle()
 void DrawManyPoints::addPoint(const Point3D &p)
 {
 	pts.push_back(p);
+	haveCachedBounds=false;
 }
 
 void DrawManyPoints::setColour(float rnew, float gnew, float bnew, float anew)
@@ -525,19 +552,11 @@ DrawDispList::~DrawDispList()
 
 }
 
-DrawableObj* DrawDispList::clone() const
-{
-	//TODO: IMPLEMENT ME
-	ASSERT(false);
-	return 0;
-}
-
-
-
 bool DrawDispList::startList(bool execute)
 {
 	//Ensure that the user has appropriately closed the list
 	ASSERT(!listActive);
+	boundBox.setInverseLimits();
 	
 	//If the list is already genned, clear it
 	if(listNum)
@@ -567,6 +586,7 @@ bool DrawDispList::endList()
 {
 	glEndList();
 
+	ASSERT(boundBox.isValid());
 	listActive=false;	
 	return (glGetError() ==0);
 }
@@ -637,27 +657,6 @@ DrawGLText::DrawGLText(std::string fontFile, unsigned int mode) : curFontMode(mo
 	alignMode = DRAWTEXT_ALIGN_LEFT;
 }
 
-DrawableObj* DrawGLText::clone() const
-{
-	DrawGLText *dt = new DrawGLText(fontString, curFontMode);
-
-	dt->origin = origin;
-	dt->r = r;
-	dt->g = g;
-	dt->b = b;
-	dt->a = a;
-	dt->alignMode=alignMode;
-
-	dt->up=up;
-	dt->textDir=textDir;
-	dt->readDir=readDir;
-	dt->isOK=isOK;
-	dt->ensureReadFromNorm=ensureReadFromNorm;
-	dt->strText=strText;
-
-	return 0;
-}
-
 void DrawGLText::draw() const
 {
 	if(!isOK)
@@ -694,13 +693,13 @@ void DrawGLText::draw() const
 
 		//---	
 		//Textdir and updir MUST be normal to one another
-		ASSERT(textDir.dotProd(up) < sqrt(std::numeric_limits<float>::epsilon()));
+		ASSERT(textDir.dotProd(up) < sqrtf(std::numeric_limits<float>::epsilon()));
 
 		//rotate around textdir cross X, if the two are not the same
 		Point3D rotateAxis;
 		Point3D newUp=up;
 		float angle=textDir.angle(Point3D(1,0,0) );
-		if(angle > sqrt(std::numeric_limits<float>::epsilon()))
+		if(angle > sqrtf(std::numeric_limits<float>::epsilon()))
 		{
 			rotateAxis = textDir.crossProd(Point3D(-1,0,0));
 			rotateAxis.normalise();
@@ -725,7 +724,7 @@ void DrawGLText::draw() const
 
 		//rotate new up direction into y around x axis
 		angle = newUp.angle(Point3D(0,1,0));
-		if(angle > sqrt(std::numeric_limits<float>::epsilon()))
+		if(angle > sqrtf(std::numeric_limits<float>::epsilon()))
 		{
 			rotateAxis = newUp.crossProd(Point3D(0,-1,0));
 			rotateAxis.normalise();
@@ -835,13 +834,14 @@ void DrawGLText::setColour(float rnew, float gnew, float bnew, float anew)
 
 void DrawGLText::getBoundingBox(BoundCube &b) const
 {
-	float minX,minY,minZ;
-	float maxX,maxY,maxZ;
 
 	if(isOK)
 	{
+		float minX,minY,minZ;
+		float maxX,maxY,maxZ;
 		font->BBox(strText.c_str(),minX,minY,minZ,maxX,maxY,maxZ);
-		b.setBounds(minX,minY,minZ,maxX,maxY,maxZ);
+		b.setBounds(minX+origin[0],minY+origin[1],minZ+origin[2],
+				maxX+origin[0],maxY+origin[1],maxZ+origin[2]);
 	}
 	else
 		b.setInverseLimits();	
@@ -887,6 +887,9 @@ void DrawRectPrism::draw() const
 	switch(drawMode)
 	{
 		case DRAW_WIREFRAME:
+		{
+			//TODO: Could speedup with LINE_STRIP/LOOP. This is 
+			//not a bottleneck atm though.
 			glLineWidth(lineWidth);	
 			glBegin(GL_LINES);
 				glColor4f(r,g,b,a);
@@ -930,6 +933,56 @@ void DrawRectPrism::draw() const
 				glVertex3f(pMin[0],pMax[1],pMin[2]);
 			glEnd();
 			break;
+		}
+		case DRAW_FLAT:
+		{
+			glBegin(GL_QUADS);
+				glColor4f(r,g,b,a);
+			
+				glNormal3f(0,0,-1);
+				//Along the bottom
+				glVertex3f(pMin[0],pMin[1],pMin[2]);
+				glVertex3f(pMin[0],pMax[1],pMin[2]);
+				glVertex3f(pMax[0],pMax[1],pMin[2]);
+				glVertex3f(pMax[0],pMin[1],pMin[2]);
+				//Up the side
+				glNormal3f(1,0,0);
+				glVertex3f(pMax[0],pMax[1],pMax[2]);
+				glVertex3f(pMax[0],pMin[1],pMax[2]);
+				glVertex3f(pMax[0],pMin[1],pMin[2]);
+				glVertex3f(pMax[0],pMax[1],pMin[2]);
+				//Over the top
+				glNormal3f(0,0,1);
+				glVertex3f(pMax[0],pMin[1],pMax[2]);
+				glVertex3f(pMax[0],pMax[1],pMax[2]);
+				glVertex3f(pMin[0],pMax[1],pMax[2]);
+				glVertex3f(pMin[0],pMin[1],pMax[2]);
+
+				//and back down
+				glNormal3f(-1,0,0);
+				glVertex3f(pMin[0],pMax[1],pMin[2]);
+				glVertex3f(pMin[0],pMin[1],pMin[2]);
+				glVertex3f(pMin[0],pMin[1],pMax[2]);
+				glVertex3f(pMin[0],pMax[1],pMax[2]);
+
+				//Now the other two sides
+				glNormal3f(0,-1,0);
+				glVertex3f(pMax[0],pMin[1],pMax[2]);
+				glVertex3f(pMin[0],pMin[1],pMax[2]);
+				glVertex3f(pMin[0],pMin[1],pMin[2]);
+				glVertex3f(pMax[0],pMin[1],pMin[2]);
+				
+				glNormal3f(0,1,0);
+				glVertex3f(pMax[0],pMax[1],pMax[2]);
+				glVertex3f(pMax[0],pMax[1],pMin[2]);
+				glVertex3f(pMin[0],pMax[1],pMin[2]);
+				glVertex3f(pMin[0],pMax[1],pMax[2]);
+
+			glEnd();
+
+			break;
+
+		}
 		default:
 			ASSERT(false);
 	}		
@@ -937,24 +990,6 @@ void DrawRectPrism::draw() const
 
 }
 
-DrawableObj* DrawRectPrism::clone() const
-{
-	DrawRectPrism *d=new DrawRectPrism;
-
-	d->pMin=pMin;
-	d->pMax=pMax;
-	d->drawMode=drawMode;
-	d->lineWidth=lineWidth;
-
-	d->r=r;
-	d->g=g;
-	d->b=b;
-	d->a=a;
-
-	d->wantsLight=false;
-	return d;
-}
-
 void DrawRectPrism::setAxisAligned( const Point3D &p1, const Point3D &p2)
 {
 	for(unsigned int ui=0; ui<3; ui++)
@@ -984,6 +1019,43 @@ void DrawRectPrism::setLineWidth(float newLineWidth)
 	lineWidth=newLineWidth;
 }
 
+void DrawRectPrism::recomputeParams(const vector<Point3D> &vecs, 
+			const vector<float> &scalars, unsigned int mode)
+{
+	switch(mode)
+	{
+		case DRAW_RECT_BIND_TRANSLATE:
+		{
+			ASSERT(vecs.size() ==1);
+			Point3D delta;
+			delta = (pMax - pMin)*0.5;
+			//Object has been translated
+			pMin = vecs[0]-delta;
+			pMax = vecs[0]+delta;
+			break;
+		}
+		case DRAW_RECT_BIND_CORNER_MOVE:
+		{
+			ASSERT(vecs.size() ==1);
+			//Delta has changed, but origin shoudl stay the same
+			Point3D mean, corner;
+			mean  = (pMin + pMax)*0.5;
+
+			//Prevent negative offset values, otherwise we can
+			//get inside out boxes
+			corner=vecs[0];
+			for(unsigned int ui=0;ui<3;ui++)
+				corner[ui]= fabs(corner[ui]);
+
+			pMin = mean-corner;
+			pMax = mean+corner;
+			break;
+		}
+		default:
+			ASSERT(false);
+	}
+}
+
 DrawTexturedQuadOverlay::DrawTexturedQuadOverlay()
 {
 	texPool=0;
@@ -1006,7 +1078,7 @@ void DrawTexturedQuadOverlay::draw() const
 		return;
 
 	ASSERT(glIsTexture(textureId));
-
+	
 	glMatrixMode(GL_PROJECTION);	
 	glPushMatrix();
 	glLoadIdentity();
@@ -1043,10 +1115,6 @@ void DrawTexturedQuadOverlay::draw() const
 	glMatrixMode(GL_MODELVIEW);
 }
 
-DrawableObj* DrawTexturedQuadOverlay::clone() const
-{
-	ASSERT(false);
-}
 
 bool DrawTexturedQuadOverlay::setTexture(const char *textureFile)
 {
@@ -1063,24 +1131,420 @@ void DrawTexturedQuadOverlay::getBoundingBox(BoundCube &b) const
 }
 
 
+DrawColourBarOverlay::DrawColourBarOverlay() 
+{
+	a=1.0;
+	string f;
+	f=getDefaultFontFile();
+	font = new FTGLPolygonFont(f.c_str());
+};
+
+
+void DrawColourBarOverlay::draw() const
+{
+	//Draw quads
+	float elemHeight;
+	//80% of bar width is for the actual colour bar itself.
+	float barWidth=0.8*width;
+	elemHeight=height/(float)rgb.size();
+	glBegin(GL_QUADS);
+	for(unsigned int ui=0;ui<rgb.size();ui++)
+	{
+		//Set the quad colour for bar element
+		glColor4f(rgb[rgb.size()-(ui+1)].v[0],
+				rgb[rgb.size()-(ui+1)].v[1],
+				rgb[rgb.size()-(ui+1)].v[2],1.0);
+
+		//draw this quad (bar element)
+		glVertex3f(tlX,tlY+(float)ui*elemHeight,0);
+		glVertex3f(tlX,tlY+(float)(ui+1)*elemHeight,0);
+		glVertex3f(tlX+barWidth,tlY+(float)(ui+1)*elemHeight,0);
+		glVertex3f(tlX+barWidth,tlY+(float)(ui)*elemHeight,0);
+	}
+
+	glEnd();
+
+	//Draw ticks on colour bar
+	glBegin(GL_LINES);
+		glColor4f(1.0,1.0,1.0f,1.0f);
+		//Top tick
+		glVertex3f(tlX,tlY,0);
+		glVertex3f(tlX+width,tlY,0);
+		//Bottom tick
+		glVertex3f(tlX,tlY+height,0);
+		glVertex3f(tlX+width,tlY+height,0);
+	glEnd();
+
+
+
+
+	if(!font || font->Error())
+	{
+#ifdef DEBUG
+		std::cerr << "Ah bugger. No font!" << std::endl;
+#endif
+		return;
+	}
+
+
+
+	//FTGL units are a pain; The devs could not decide
+	//whether to implement them in opengl coords or real coords
+	//so they did neither, and implemented them in "points".
+	//here we assume that we can transform 1 ftgl unit
+	//to 1 opengl unit by inversion 
+	const float FTGL_DEFAULT_UNIT_SCALE=1.0/72.0;
+
+	glColor3f(1.0f,1.0f,1.0f);	
+	font->FaceSize(3);
+	glDisable(GL_CULL_FACE);
+	glPushMatrix();
+	glTranslatef(tlX+width,tlY,0);
+	string s;
+	stream_cast(s,max);
+	//Note negative sign to flip from y-down screen (opengl) to text dir
+	//(y up)
+	glScaled(FTGL_DEFAULT_UNIT_SCALE,
+			-FTGL_DEFAULT_UNIT_SCALE,FTGL_DEFAULT_UNIT_SCALE);
+	font->Render(s.c_str());
+	glPopMatrix();
+
+	glPushMatrix();
+	glTranslatef(tlX+width,tlY+height,0);
+	stream_cast(s,min);
+	//Note negative sign to flip from y-down screen (opengl) to text dir
+	//(y up)
+	glScaled(FTGL_DEFAULT_UNIT_SCALE,
+			-FTGL_DEFAULT_UNIT_SCALE,FTGL_DEFAULT_UNIT_SCALE);
+	font->Render(s.c_str());
+	glPopMatrix();
+	glEnable(GL_CULL_FACE);
+
+
+
+}
+
+void DrawColourBarOverlay::setColourVec(const vector<float> &r,
+					const vector<float> &g,
+					const vector<float> &b)
+{
+	ASSERT(r.size() == g.size());
+	ASSERT(g.size() == b.size());
+	rgb.resize(r.size());
+	for(unsigned int ui=0;ui<r.size();ui++)
+	{
+		rgb[ui].v[0]=r[ui];
+		rgb[ui].v[1]=g[ui];
+		rgb[ui].v[2]=b[ui];
+	}
+
+
+}
+
+void DrawColourBarOverlay::getBoundingBox(BoundCube &b) const
+{
+	b.setInvalid();
+}
+
+
+DrawField3D::DrawField3D() : alphaVal(0.2f), pointSize(1.0f), drawBoundBox(true),volumeGrid(false), volumeRenderMode(0), field(0) 
+{
+	boxColour.r = boxColour.g = boxColour.b = boxColour.a = 1.0f;
+	ptsCacheOK=false;
+}
+
+DrawField3D::~DrawField3D()
+{
+	if(field)
+		delete field;
+}
+
+
+void DrawField3D::getBoundingBox(BoundCube &b) const
+{
+	ASSERT(field)
+	b.setBounds(field->getMinBounds(),field->getMaxBounds());
+}
+
+
+void DrawField3D::setField(const Voxels<float> *newField)
+{
+	field=newField;
+}
+
+void DrawField3D::setRenderMode(unsigned int mode)
+{
+	volumeRenderMode=mode;
+}
+
+void DrawField3D::setColourMinMax()
+{
+	colourMapBound[0]=field->min();
+	colourMapBound[1]=field->max();
+
+	ASSERT(colourMapBound[0] <=colourMapBound[1]);
+}
+
+			
+void DrawField3D::draw() const
+{
+
+	ASSERT(field);
+
+	//Depend upon the render mode
+	switch(volumeRenderMode)
+	{
+		case VOLUME_POINTS:
+		{
+			size_t fieldSizeX,fieldSizeY,fieldSizeZ;
+			Point3D p;
+
+			field->getSize(fieldSizeX,fieldSizeY, fieldSizeZ);
+
+			//We need to generate some points, then sort them by distance
+			//from eye (back to front), otherwise they will not blend properly
+			std::vector<std::pair<float,unsigned int >  > eyeDists;
+
+			Point3D camOrigin = curCam->getOrigin();
+
+			if(!ptsCacheOK)
+			{
+				ptsCache.clear();
+				for(unsigned int uiX=0; uiX<fieldSizeX; uiX++)
+				{
+					for(unsigned int uiY=0; uiY<fieldSizeY; uiY++)
+					{
+						for(unsigned int uiZ=0; uiZ<fieldSizeZ; uiZ++)
+						{
+							float v;
+							v=field->getData(uiX,uiY,uiZ);
+							if(v > std::numeric_limits<float>::epsilon())
+							{
+								RGBThis rgb;
+								//Set colour and point loc
+								colourMapWrap(colourMapID,rgb.v,
+										field->getData(uiX,uiY,uiZ), 
+										colourMapBound[0],colourMapBound[1]);
+								
+								ptsCache.push_back(make_pair(field->getPoint(uiX,uiY,uiZ),rgb));
+							}
+						}
+					}
+				}
+					
+
+				ptsCacheOK=true;
+			}
+			eyeDists.resize(ptsCache.size());
+			
+			//Set up an original index for the eye distances
+			#pragma omp parallel for
+			for(unsigned int ui=0;ui<ptsCache.size();ui++)
+			{
+				eyeDists[ui].first=ptsCache[ui].first.sqrDist(camOrigin);
+				eyeDists[ui].second=ui;
+			}
+
+			ComparePairFirstReverse cmp;
+			std::sort(eyeDists.begin(),eyeDists.end(),cmp);	
+
+			//render each element in the field as a point
+			//the colour of the point is determined by its scalar value
+			glPointSize(pointSize);
+			glBegin(GL_POINTS);
+			for(unsigned int ui=0;ui<ptsCache.size();ui++)
+			{
+				unsigned int idx;
+				idx=eyeDists[ui].second;
+				//Tell openGL about it
+				glColor4f(((float)(ptsCache[idx].second.v[0]))/255.0f, 
+						((float)(ptsCache[idx].second.v[1]))/255.0f,
+						((float)(ptsCache[idx].second.v[2]))/255.0f, 
+						alphaVal);
+				glVertex3f(ptsCache[idx].first[0],ptsCache[idx].first[1],ptsCache[idx].first[2]);
+			}
+			glEnd();
+			break;
+		}
+
+		default:
+			//Not implemented
+			ASSERT(false); 
+	}
+
+	//Draw the bounding box as required
+	if(drawBoundBox)
+	{
+		Point3D pMin,pMax;
+		pMin=field->getMinBounds();
+		pMax=field->getMaxBounds();
+	
+
+		glColor4f(boxColour.r, boxColour.g,boxColour.b,boxColour.a);
+		//Draw lines between field min and max
+		glBegin(GL_LINES);
+			//Bottom corner out
+			glVertex3f(pMin[0],pMin[1],pMin[2]);
+			glVertex3f(pMax[0],pMin[1],pMin[2]);
+			
+			glVertex3f(pMin[0],pMin[1],pMin[2]);
+			glVertex3f(pMin[0],pMax[1],pMin[2]);
+
+			glVertex3f(pMin[0],pMin[1],pMin[2]);
+			glVertex3f(pMin[0],pMin[1],pMax[2]);
+			
+			//Top Corner out
+			glVertex3f(pMax[0],pMax[1],pMax[2]);
+			glVertex3f(pMin[0],pMax[1],pMax[2]);
+		
+			glVertex3f(pMax[0],pMax[1],pMax[2]);
+			glVertex3f(pMax[0],pMin[1],pMax[2]);
+			
+			glVertex3f(pMax[0],pMax[1],pMax[2]);
+			glVertex3f(pMax[0],pMax[1],pMin[2]);
+
+			//Missing pieces - in a across down across shape
+			glVertex3f(pMin[0],pMax[1],pMin[2]);
+			glVertex3f(pMax[0],pMax[1],pMin[2]);
+			
+			glVertex3f(pMax[0],pMax[1],pMin[2]);
+			glVertex3f(pMax[0],pMin[1],pMin[2]);
+
+			glVertex3f(pMax[0],pMin[1],pMin[2]);
+			glVertex3f(pMax[0],pMin[1],pMax[2]);
+			
+			glVertex3f(pMax[0],pMin[1],pMax[2]);
+			glVertex3f(pMin[0],pMin[1],pMax[2]);
+			
+			glVertex3f(pMin[0],pMin[1],pMax[2]);
+			glVertex3f(pMin[0],pMax[1],pMax[2]);
+
+			glVertex3f(pMin[0],pMax[1],pMax[2]);
+			glVertex3f(pMin[0],pMax[1],pMin[2]);
+		glEnd();
+	}
+	//Draw the projections
+}
+
+void DrawField3D::setAlpha(float newAlpha)
+{
+	alphaVal=newAlpha;
+}
+
+void DrawField3D::setPointSize(float size)
+{
+	pointSize=size;
+}
+
+void DrawField3D::setMapColours(unsigned int mapID)
+{
+	ASSERT(mapID < NUM_COLOURMAPS);
+	colourMapID= mapID;
+}
+
+void DrawField3D::setBoxColours(float rNew, float gNew, float bNew, float aNew)
+{
+	boxColour.r = rNew;
+	boxColour.g = gNew;
+	boxColour.b = bNew;
+	boxColour.a = aNew;
+
+}
+
+
+DrawIsoSurface::DrawIsoSurface()
+{
+	cacheOK=false;
+	drawMode=DRAW_SMOOTH;
+	threshold=0.5;
+	voxels=0;
+	
+	r=0.5;
+	g=0.5;
+	b=0.5;
+	a=1.0;	
+}
+
+DrawIsoSurface::~DrawIsoSurface()
+{
+	if(voxels)
+		delete voxels;
+}
+
+void DrawIsoSurface::swapVoxels(Voxels<float> *f)
+{
+	std::swap(f,voxels);
+	cacheOK=false;
+	mesh.clear();
+}
+
+
+void DrawIsoSurface::updateMesh() const
+{
+
+	mesh.clear();
+	marchingCubes(*voxels, threshold,mesh);
+
+	cacheOK=true;
+
+}
+
+void DrawIsoSurface::getBoundingBox(BoundCube &b) const
+{
+	if(voxels)
+	{
+		b.setBounds(voxels->getMinBounds(),
+				voxels->getMaxBounds());
+	}
+	else
+		b.setInverseLimits();
+}
+
+void DrawIsoSurface::draw() const
+{
+
+	if(!cacheOK)
+	{
+		//Hmm, we don't have a cached copy of the isosurface mesh.
+		//we will need to compute one, it would seem.
+		updateMesh();
+	}
+
+	//This could be optimised by using triangle strips
+	//rather than direct triangles.
+	glColor4f(r,g,b,a);
+	glPushAttrib(GL_CULL_FACE);
+	glDisable(GL_CULL_FACE);	
+	glBegin(GL_TRIANGLES);	
+	for(unsigned int ui=0;ui<mesh.size();ui++)
+	{
+		glNormal3fv(mesh[ui].normal[0].getValueArr());
+		glVertex3fv(mesh[ui].p[0].getValueArr());
+		glNormal3fv(mesh[ui].normal[1].getValueArr());
+		glVertex3fv(mesh[ui].p[1].getValueArr()),
+		glNormal3fv(mesh[ui].normal[2].getValueArr());
+		glVertex3fv(mesh[ui].p[2].getValueArr());
+	}
+	glEnd();
+	glPopAttrib();
+}
+
+
 #ifdef HPMC_GPU_ISOSURFACE
 
 //HPMC on GPU Isosurface code is GPL
 // See class definition for licence 
 
-//TODO: REmove me
-using std::cerr;
-using std::endl;
-
 
 DrawIsoSurfaceWithShader::DrawIsoSurfaceWithShader()
 {
+	dataset=0;
 	shadersOK=false;
 	wireframe=false;
 		
 	//TODO: Make this static, once-only?? Have some global drawinit code?
 	GLenum err = glewInit();
-	shadersOK= (GLEW_OK != err) &&(GLEW_ARB_vertex_program);
+	shadersOK= (GLEW_OK == err) &&(GLEW_ARB_vertex_program);
 	
 	//Create shaders
 	 shaded_vertex_shader =
@@ -1119,20 +1583,59 @@ DrawIsoSurfaceWithShader::DrawIsoSurfaceWithShader()
 		"    gl_FrontColor = gl_Color;\n"
 		"}\n";
 
+
 }
 
 
 DrawIsoSurfaceWithShader::~DrawIsoSurfaceWithShader()
 {
+	if(dataset)
+		delete[] dataset;
 }
 
 
-bool DrawIsoSurfaceWithShader::init(unsigned int dataX,
-		unsigned int dataY, unsigned int dataZ, const char *dataset)
+
+void DrawIsoSurfaceWithShader::getBoundingBox(BoundCube &b) const
+{
+	b=bounds;
+}
+
+void DrawIsoSurfaceWithShader::setScalarThresh(float thresh)
+{
+	threshold=(float)(thresh/trueMax*255.0f);
+}
+
+
+bool DrawIsoSurfaceWithShader::init(const Voxels<float> &f)
 {
 	if(!shadersOK)
 		return false;
 
+	unsigned long long nX,nY,nZ;
+
+	f.getSize(nX,nY,nZ);
+	try
+	{
+		dataset = new char[nX*nY*nZ];
+	}
+	catch(std::bad_alloc)
+	{
+		return false;
+	}
+
+	//This algorithm does not handle voxel fields with negative values
+	ASSERT(f.min() >= 0.0f);
+	trueMax=f.max();
+
+	bounds.setBounds(f.getMinBounds(),f.getMaxBounds());
+
+	for(unsigned int ui=0;ui<nX*nY*nZ; ui++)
+		dataset[ui]=(float)(f.getData(ui)/trueMax*255.0f);
+
+	volume_size_x=nX;
+	volume_size_y=nY;
+	volume_size_z=nZ;
+
 	// --- upload volume ------------------------------------------------------
 
 	GLint alignment;
@@ -1245,7 +1748,6 @@ void DrawIsoSurfaceWithShader::linkProgram( GLuint program, const std::string& w
 	GLint linkstatus;
 	glGetProgramiv( program, GL_LINK_STATUS, &linkstatus );
 	if ( linkstatus != GL_TRUE ) {
-		cerr << "Linking of " << what << " failed, infolog:" << endl;
 
 		GLint logsize;
 		glGetProgramiv( program, GL_INFO_LOG_LENGTH, &logsize );
@@ -1253,10 +1755,6 @@ void DrawIsoSurfaceWithShader::linkProgram( GLuint program, const std::string& w
 		if ( logsize > 0 ) {
 			GLchar *infolog = new char[ logsize+1 ];
 			glGetProgramInfoLog( program, logsize, NULL, infolog );
-			cerr << infolog << endl;
-		}
-		else {
-			cerr << "Empty log message" << endl;
 		}
 		shadersOK=false;
 	}
@@ -1268,6 +1766,7 @@ void DrawIsoSurfaceWithShader::draw() const
 	ASSERT(shadersOK);
 	HPMCbuildHistopyramid( hpmc_h, threshold );
 
+	//TODO: FIXME:  Does not work??! Go back to examples and look again
 		
 	// --- render surface ------------------------------------------------------
 	if ( !wireframe ) {
@@ -1293,7 +1792,6 @@ void DrawIsoSurfaceWithShader::compileShader( GLuint shader, const std::string&
 	GLint compile_status;
 	glGetShaderiv( shader, GL_COMPILE_STATUS, &compile_status );
 	if ( compile_status != GL_TRUE ) {
-		cerr << "Compilation of " << what << " failed, infolog:" << endl;
 
 		GLint logsize;
 		glGetShaderiv( shader, GL_INFO_LOG_LENGTH, &logsize );
@@ -1301,13 +1799,160 @@ void DrawIsoSurfaceWithShader::compileShader( GLuint shader, const std::string&
 		if ( logsize > 0 ) {
 			GLchar *infolog = new char[ logsize+1 ];
 			glGetProgramInfoLog( shader, logsize, NULL, infolog );
-			cerr << infolog << endl;
 		}
 		else {
-			cerr << "Empty log message" << endl;
 		}
 		shadersOK=false;
 	}
 }
-
 #endif
+
+
+DrawAxis::DrawAxis()
+{
+}
+
+DrawAxis::~DrawAxis()
+{
+}
+
+void DrawAxis::setStyle(unsigned int s)
+{
+	style=s;
+}
+
+void DrawAxis::setSize(float s)
+{
+	size=s;
+}
+
+void DrawAxis::setPosition(const Point3D &p)
+{
+	position=p;
+}
+
+void DrawAxis::draw() const
+
+{
+	float halfSize=size/2.0f;
+	glPushAttrib(GL_LIGHTING_BIT);
+	glDisable(GL_LIGHTING);
+	glBegin(GL_LINES);
+	//Draw lines
+	glColor3f(1.0f,0.0f,0.0f);
+	glVertex3f(position[0]-halfSize,
+	           position[1],position[2]);
+	glVertex3f(position[0]+halfSize,
+	           position[1],position[2]);
+
+	glColor3f(0.0f,1.0f,0.0f);
+	glVertex3f(position[0],
+	           position[1]-halfSize,position[2]);
+	glVertex3f(position[0],
+	           position[1]+halfSize,position[2]);
+
+	glColor3f(0.0f,0.0f,1.0f);
+	glVertex3f(position[0],
+	           position[1],position[2]-halfSize);
+	glVertex3f(position[0],
+	           position[1],position[2]+halfSize);
+	glEnd();
+	glPopAttrib();
+
+
+
+	float numSections=20.0f;
+	float twoPi = 2.0f *M_PI;
+	float radius = 0.1*halfSize;
+	//Draw axis cones
+
+	//+x
+	glPushMatrix();
+	glTranslatef(position[0]+halfSize,position[1],position[2]);
+
+	glColor3f(1.0f,0.0f,0.0f);
+	glBegin(GL_TRIANGLE_FAN);
+	glVertex3f(radius,0,0);
+	glNormal3f(1,0,0);
+	for (unsigned int i = 0; i<=numSections; i++)
+	{
+		glVertex3f(0,radius * cos(i *  twoPi / numSections),
+		           radius* sin(i * twoPi / numSections));
+		glNormal3f(0,cos(i*twoPi/numSections),sin(i*twoPi/numSections));
+	}
+	glEnd();
+	glBegin(GL_TRIANGLE_FAN);
+	glVertex3f(0,0,0);
+	glNormal3f(-1,0,0);
+	for (unsigned int i = 0; i<=numSections; i++)
+	{
+		glVertex3f(0,-radius * cos(i *  twoPi / numSections),
+		           radius* sin(i * twoPi / numSections));
+		glNormal3f(-1,0,0);
+	}
+	glEnd();
+	glPopMatrix();
+
+	//+y
+	glColor3f(0.0f,1.0f,0.0f);
+	glPushMatrix();
+	glTranslatef(position[0],position[1]+halfSize,position[2]);
+	glBegin(GL_TRIANGLE_FAN);
+	glVertex3f(0,radius,0);
+	glNormal3f(0,1,0);
+	for (unsigned int i = 0; i<=numSections; i++)
+	{
+		glVertex3f(radius * sin(i *  twoPi / numSections),0,
+		           radius* cos(i * twoPi / numSections));
+		glNormal3f(sin(i*twoPi/numSections),0,cos(i*twoPi/numSections));
+	}
+	glEnd();
+
+	glBegin(GL_TRIANGLE_FAN);
+	glVertex3f(0,0,0);
+	glNormal3f(0,-1,0);
+	for (unsigned int i = 0; i<=numSections; i++)
+	{
+		glVertex3f(radius * cos(i *  twoPi / numSections),0,
+		           radius* sin(i * twoPi / numSections));
+		glNormal3f(0,-1,0);
+	}
+	glEnd();
+
+	glPopMatrix();
+
+
+
+	//+z
+	glColor3f(0.0f,0.0f,1.0f);
+	glPushMatrix();
+	glTranslatef(position[0],position[1],position[2]+halfSize);
+	glBegin(GL_TRIANGLE_FAN);
+	glVertex3f(0,0,radius);
+	glNormal3f(0,0,1);
+	for (unsigned int i = 0; i<=numSections; i++)
+	{
+		glVertex3f(radius * cos(i *  twoPi / numSections),
+		           radius* sin(i * twoPi / numSections),0);
+		glNormal3f(cos(i*twoPi/numSections),sin(i*twoPi/numSections),0);
+	}
+	glEnd();
+
+	glBegin(GL_TRIANGLE_FAN);
+	glVertex3f(0,0,0);
+	glNormal3f(0,0,-1);
+	for (unsigned int i = 0; i<=numSections; i++)
+	{
+		glVertex3f(-radius * cos(i *  twoPi / numSections),
+		           radius* sin(i * twoPi / numSections),0);
+		glNormal3f(0,0,-1);
+	}
+	glEnd();
+	glPopMatrix();
+}
+
+void DrawAxis::getBoundingBox(BoundCube &b) const
+{
+	b.setInvalid();
+}
+
