@@ -23,12 +23,13 @@
 //Note, this define is repeated in drawables.cpp
 //to avoid exposing glew.h in this header,
 //which complains bitterly about header orders.
-//#define HPMC_GPU_ISOSURFACE 0
+//#define HPMC_GPU_ISOSURFACE
 
-#include "APTClasses.h"
 #include "cameras.h"
 #include "textures.h"
-
+#include "voxels.h"
+#include "isoSurface.h"
+#include "basics.h"
 
 //STL includes
 #include <vector>
@@ -47,7 +48,7 @@
 //TODO: Work out if there is any way of obtaining the maximum 
 //number of items that can be drawn in an opengl context
 //For now Max it out at 10 million (~120MB of vertex data)
-const size_t MAX_NUM_DRAWABLE_POINTS=10000000;
+const size_t MAX_NUM_DRAWABLE_POINTS=10;
 
 
 //OK, the new FTGL is fucked up. It actually uses defines from
@@ -75,14 +76,39 @@ enum
 };
 
 
-//TODO: Make sure all solids support this (how to tell glu??)
 //!Primitve drawing mode. (wireframe/solid)
 enum
 {
 	DRAW_WIREFRAME,
+	DRAW_FLAT,
+	DRAW_SMOOTH,
+	DRAW_END_ENUM, //Not a mode, just a marker to catch end-of-enum
 };
 
-#include "basics.h"
+//!Axis styles
+enum
+{
+	AXIS_IN_SPACE,	
+};
+
+
+//!Binding enums. Needed to bind drawable selection
+//to internal modification actions inside the drawable
+enum
+{
+	DRAW_SPHERE_BIND_ORIGIN,
+	DRAW_SPHERE_BIND_RADIUS,
+	DRAW_VECTOR_BIND_ORIENTATION,
+	DRAW_VECTOR_BIND_ORIGIN,
+	DRAW_CYLINDER_BIND_ORIGIN,
+	DRAW_CYLINDER_BIND_DIRECTION,
+	DRAW_CYLINDER_BIND_RADIUS,
+	DRAW_RECT_BIND_TRANSLATE,
+	DRAW_RECT_BIND_CORNER_MOVE,
+	DRAW_BIND_ENUM_END,
+};
+
+
 
 //!An abstract bas class for drawing primitives
 class DrawableObj
@@ -98,11 +124,12 @@ class DrawableObj
 		//!Wants lighting calculations active during render?
 		bool wantsLight;
 
+		//!Is this an overlay? By default, no
+		virtual bool isOverlay() const { return false;}
+
 		//!Constructor
 		DrawableObj();
 		
-		//!Clone object
-		virtual DrawableObj* clone() const = 0;
 
 		//!Set the active state of the object
 		void setActive(bool active);
@@ -115,6 +142,9 @@ class DrawableObj
 		virtual void getBoundingBox(BoundCube &b) const = 0;
 		//!Drawable destructor
 		virtual ~DrawableObj();
+
+		//!If we offer any kind of external pointer interface; use this to do a recomputation as needed. This is needed for selection binding behaviour
+		virtual void recomputeParams(const vector<Point3D> &vecs, const vector<float> &scalars, unsigned int mode) {};
 };
 
 //A single point drawing class 
@@ -133,9 +163,6 @@ class DrawPoint : public DrawableObj
 		//!Destructor
 		virtual ~DrawPoint();
 	
-		//!Clone object	
-		DrawableObj* clone() const;
-		
 		//!Sets the color of the point to be drawn
 		void setColour(float r, float g, float b, float alpha);
 		//!Draws the points
@@ -156,21 +183,20 @@ class DrawManyPoints : public DrawableObj
 		float r,g,b,a;
 		//!Size of the point
 		float size;
+
+		mutable bool haveCachedBounds;
+		mutable BoundCube cachedBounds;
 	public:
 		//!Constructor
 		DrawManyPoints();
 		//!Destructor
 		virtual ~DrawManyPoints();
-		//!Clone object	
-		DrawableObj* clone() const;
 		//!Swap out the internal vector with an extenal one
 		void swap(std::vector<Point3D> &);
 		//!Remove all points
 		void clear();
 		//!Add points into the drawing vector
 		void addPoints(const std::vector<Point3D> &);
-		//!Add points into the drawing vector
-		void addPoints(const std::vector<IonHit> &);
 		//!Add a single point into the drawing vector
 		void addPoint(const Point3D &);
 		//! set the color of the points to be drawn
@@ -183,7 +209,7 @@ class DrawManyPoints : public DrawableObj
 		//!Shuffle the points to remove anisotropic drawing effects. Thus must be done prior to draw call.
 		void shuffle();
 		//!Get the bounding box that encapuslates this object
-		void getBoundingBox(BoundCube &b) const { b.setBounds(pts);}
+		void getBoundingBox(BoundCube &b) const; 
 		
 		//!return number of points
 		size_t getNumPts() { return pts.size();};
@@ -211,8 +237,6 @@ class DrawVector: public DrawableObj
 		//!Destructor
 		virtual ~DrawVector();
 	
-		//!Clone object	
-		DrawableObj* clone() const;
 		
 		//!Sets the color of the point to be drawn
 		void setColour(float r, float g, float b, float alpha);
@@ -222,15 +246,14 @@ class DrawVector: public DrawableObj
 		void setOrigin(const Point3D &);
 		//!Sets the location of the poitns
 		void setVector(const Point3D &);
+		//!Gets the cylinder axis direction
+		Point3D getVector(){ return vector;};
 
 		void getBoundingBox(BoundCube &b) const; 
+		//!Recompute the internal parameters using the input vector information
+		void recomputeParams(const std::vector<Point3D> &vecs, 
+					const std::vector<float> &scalars, unsigned int mode);
 
-		//!Return a pointer to the origin. this breaks data encapsulation
-		//so use with care!
-		Point3D *originPtr(){return &origin;};
-		//!Return a pointer to the origin. this breaks data encapsulation
-		//so use with care!
-		Point3D *directionPtr(){return &vector;};
 };
 
 //! A single colour triangle
@@ -239,6 +262,7 @@ class DrawTriangle : public DrawableObj
 	protected:
 		//!The vertices of the triangle
 		Point3D vertices[3];
+		Point3D vertNorm[3];
 		//!Colour data - red, green, blue, alpha
 		float r,g,b,a;
 	public:
@@ -246,11 +270,11 @@ class DrawTriangle : public DrawableObj
 		DrawTriangle();
 		//!Destructor
 		virtual ~DrawTriangle();
-		//!Clone object	
-		DrawableObj* clone() const;
 
 		//!Set one of three vertices (0-2) locations
 		void setVertex(unsigned int, const Point3D &);
+		//!Set the vertex normals
+		void setVertexNorm(unsigned int, const Point3D &);
 		//!Set the colour of the triangle
 		void setColour(float r, float g, float b, float a);
 		//!Draw the triangle
@@ -281,8 +305,6 @@ class DrawQuad : public DrawableObj
 		DrawQuad();
 		//!Destructor
 		virtual ~DrawQuad();
-		//!Clone object	
-		DrawableObj* clone() const;
 		//!sets the vertices to defautl colours (r g b and white ) for each vertex respectively
 		void colourVerticies();
 		//!Set vertex's location
@@ -319,17 +341,19 @@ class DrawSphere : public DrawableObj
 		DrawSphere();
 		//! Destructor
 		virtual ~DrawSphere();
-		//!Clone object	
-		DrawableObj* clone() const;
 
 		//!Sets the location of the sphere's origin
 		void setOrigin(const Point3D &p);
+		//!Gets the location of the sphere's origin
+		Point3D getOrigin() const { return origin;};
 		//!Set the number of lateral segments
 		void setLatSegments(unsigned int);
 		//!Set the number of longitudinal segments
 		void setLongSegments(unsigned int);
 		//!Set the radius
 		void setRadius(float);
+		//!get the radius
+		float getRadius() const { return radius;};
 		//!Set the colour (rgba) of the object
 		void setColour(float r,float g,float b,float a);
 		//!Draw the sphere
@@ -337,13 +361,8 @@ class DrawSphere : public DrawableObj
 		//!Get the bounding box that encapuslates this object
 		void getBoundingBox(BoundCube &b) const ;
 
-		//!Return a pointer to the radius. this breaks data encapsulation
-		//so use with care!
-		float *radiusPtr(){return &radius;};
-
-		//!Return a pointer to the origin. this breaks data encapsulation
-		//so use with care!
-		Point3D *originPtr(){return &origin;};
+		//!Recompute the internal parameters using the input vector information
+		void recomputeParams(const vector<Point3D> &vecs, const vector<float> &scalars, unsigned int mode);
 
 };
 
@@ -374,8 +393,6 @@ class DrawCylinder : public DrawableObj
 		DrawCylinder();
 		//!Destructor
 		virtual ~DrawCylinder();
-		//!Clone object	
-		DrawableObj* clone() const;
 
 		//!Set the location of the base of the cylinder
 		void setOrigin(const Point3D &pt);
@@ -384,8 +401,14 @@ class DrawCylinder : public DrawableObj
 		//!Number of cuts  along axis - ie segments
 		void setStacks(unsigned int i);
 
+		//!Gets the location of the origin
+		Point3D getOrigin(){ return origin;};
+		//!Gets the cylinder axis direction
+		Point3D getDirection(){ return direction;};
 		//!Set end radius
 		void setRadius(float val);
+		//!get the radius
+		float getRadius() const { return radius;};
 		//!Set the orientation of cylinder
 		void setDirection(const Point3D &pt);
 		//!Set the length of cylinder
@@ -398,14 +421,8 @@ class DrawCylinder : public DrawableObj
 		//!Get the bounding box that encapuslates this object
 		void getBoundingBox(BoundCube &b) const ;
 
-		//!Get radius pointer (radii must be locked)
-		float *radiusPtr();
-	
-		//!Get the origin poitner	
-		Point3D *originPtr();
-	
-		//!Get the direction pointer	
-		Point3D *directionPtr();
+		//!Recompute the internal parameters using the input vector information
+		void recomputeParams(const vector<Point3D> &vecs, const vector<float> &scalars, unsigned int mode);
 
 		//!Lock (or unlock) the radius to the start radius (i.e. synch the two)
 		void lockRadii(bool doLock=true) {radiiLocked=doLock;};
@@ -437,8 +454,6 @@ class DrawDispList : public DrawableObj
 		DrawDispList();
 		//!Destructor
 		virtual ~DrawDispList();
-		//!Clone object	
-		DrawableObj* clone() const;
 
 		//!Execute the display list
 		void draw() const;		
@@ -521,20 +536,18 @@ class DrawGLText : public DrawableObj
 	public:
 		//!Constructor (font file & text mode)
 		/* Valid font types are FTGL_x where x is
-		 * 	\* BITMAP
-		 * 	\* PIXMAP
-		 * 	\* OUTLINE
-		 * 	\* POLYGON
-		 * 	\* EXTRUDE
-		 * 	\* TEXTURE
+		 * 	BITMAP
+		 * 	PIXMAP
+		 * 	OUTLINE
+		 * 	POLYGON
+		 * 	EXTRUDE
+		 * 	 TEXTURE
 		 */
 		DrawGLText(std::string fontFile,
 					unsigned int ftglTextMode);
 
 		//!Destructor
 		virtual ~DrawGLText();
-		//!Clone object	
-		DrawableObj* clone() const;
 		
 		//!Set the camera used to draw scene
 		static void setCurCamera(const Camera *);
@@ -621,9 +634,9 @@ class DrawRectPrism  : public DrawableObj
 
 		//!Draw object
 		void draw() const;
-		//!Clone object
-		DrawableObj *clone() const; 
 
+		//!Set the draw mode
+		void setDrawMode(unsigned int n) { drawMode=n;};
 		//!Set colour of box
 		void setColour(float rnew, float gnew, float bnew, float anew);
 		//!Set thickness of box
@@ -634,8 +647,52 @@ class DrawRectPrism  : public DrawableObj
 		void setAxisAligned(const BoundCube &b);
 
 		void getBoundingBox(BoundCube &b) const;
+		
+		//!Recompute the internal parameters using the input vector information
+		void recomputeParams(const vector<Point3D> &vecs, const vector<float> &scalars, unsigned int mode);
 };
 
+struct RGBFloat
+{
+	float v[3];
+};
+
+class DrawColourBarOverlay : public DrawableObj
+{
+	private:
+		FTFont *font;
+
+		//!Colours for each element
+		vector<RGBFloat> rgb;
+		//alpha (transparancy) value
+		float a;
+		//!Minimum and maximum values for the colour bar (for ticks)
+		float min,max;
+		//!Height and width of bar (total)
+		float height,width;
+		//!top left of bar
+		float tlX,tlY;
+
+	public:
+	
+		DrawColourBarOverlay();
+		~DrawColourBarOverlay(){delete font;};
+		void getBoundingBox(BoundCube &b) const ;
+
+		//!This is an overlay
+		bool isOverlay() const {return true;};
+		void setColourVec(const vector<float> &r,
+					const vector<float> &g,
+					const vector<float> &b);
+		//!Draw object
+		void draw() const;
+
+		void setAlpha(float alpha) { a=alpha;};
+		void setSize(float widthN, float heightN) {height=heightN, width=widthN;} 
+		void setPosition(float newTLX,float newTLY) { tlX=newTLX; tlY=newTLY;}
+		void setMinMax(float minNew,float maxNew) { min=minNew;max=maxNew;};
+		
+};
 
 //!A class to hande textures to draw
 class DrawTexturedQuadOverlay : public DrawableObj
@@ -652,6 +709,8 @@ class DrawTexturedQuadOverlay : public DrawableObj
 		DrawTexturedQuadOverlay();
 		~DrawTexturedQuadOverlay();
 
+		//!This is an overlay
+		bool isOverlay() const {return true;};
 	
 		void setWindowSize(unsigned int x, unsigned int y){winX=x;winY=y;};	
 
@@ -668,13 +727,141 @@ class DrawTexturedQuadOverlay : public DrawableObj
 
 		//!Draw object
 		void draw() const;
-		//!Clone object
-		DrawableObj *clone() const; 
 
 		void getBoundingBox(BoundCube &b) const ;
 };
 
 
+struct RGBThis
+{
+	unsigned char v[3];
+};
+
+//!This class allows for the visualisation of 3D scalar fields
+class DrawField3D : public DrawableObj
+{
+	private:
+		mutable std::vector<std::pair<Point3D,RGBThis> > ptsCache;
+		mutable bool ptsCacheOK;
+	protected:
+		//CurCam
+		static const Camera *curCam;
+		//!Alpha transparancy of objects in field
+		float alphaVal;
+
+		//!Size of points in the field -
+		//only meaningful if the render mode is set to alpha blended points
+		float pointSize;
+
+		//!True if the scalar field's bounding box is to be drawn
+		bool drawBoundBox;
+
+		//!Colours for the bounding boxes
+		Colour boxColour;
+		
+		//!True if volume grid is enabled
+		bool volumeGrid;
+
+		//!Colour map lower and upper bounds
+		float colourMapBound[2];
+
+		//!Which colourmap to use
+		unsigned int colourMapID;
+
+		//!Sets the render mode for the 3D volume 
+		/* Possible modes
+		 * 0: Alpha blended points
+		 */
+		unsigned int volumeRenderMode;
+		//!The scalar field - used to store data values
+		const Voxels<float> *field;
+	public:
+		//!Default Constructor
+		DrawField3D();
+		//!Destructor
+		virtual ~DrawField3D();
+
+
+		//!Get the bounding box for this object
+		void getBoundingBox(BoundCube &b) const;
+		
+		//!Set the render mode (see volumeRenderMode variable for details)
+		void setRenderMode(unsigned int);
+		
+		//!Set the field pointer 
+		void setField(const Voxels<float> *field); 
+
+		//!Set the alpha value for elemnts
+		void setAlpha(float alpha);
+
+		//!Set the colour bar minima and maxima from current field values
+		void setColourMinMax();
+
+		//!Set the colourMap ID
+		void setColourMapID(unsigned int i){ colourMapID=i;};
+
+		//!Render the field
+		void draw() const;
+
+		//!Set the size of points
+		void setPointSize(float size);
+		
+		//!Set the colours that ar ebeing used in the tempMap
+		void setMapColours(unsigned int map);
+
+		//!Set the coour of the bounding box
+		void setBoxColours(float r, float g, float b, float a);
+		//!Set the current camera
+		static void setCurCamera(const Camera *c){curCam=c;};
+
+};
+
+class DrawIsoSurface: public DrawableObj
+{
+private:
+	mutable bool cacheOK;
+
+	//!should we draw the thing 
+	//	- in wireframe
+	//	-Flat
+	//	-Smooth
+	//
+	unsigned int drawMode;
+
+	//!Isosurface scalar threshold
+	float threshold;
+
+	Voxels<float> *voxels;	
+
+	mutable vector<TriangleWithVertexNorm> mesh;
+
+	//!Warning. Although I declare this as const, I do some naughty mutating to the cache.
+	void updateMesh() const;	
+	
+	//!Point colour (r,g,b,a) range: [0.0f,1.0f]
+	float r,g,b,a;
+public:
+
+	DrawIsoSurface();
+	~DrawIsoSurface();
+
+	//!Transfer ownership of data pointer to class
+	void swapVoxels(Voxels<float> *v);
+
+	//!Set the drawing method
+
+	//Draw
+	void draw() const;
+
+	//!Set the isosurface value
+	void setScalarThresh(float thresh) { threshold=thresh;cacheOK=false;mesh.clear();};
+
+	//!Get the bouding box (of the entire scalar field)	
+	void getBoundingBox(BoundCube &b) const ;
+		
+	//!Sets the color of the point to be drawn
+	void setColour(float rP, float gP, float bP, float alpha) { r=rP;g=gP;b=bP;a=alpha;} ;
+};
 
 #ifdef HPMC_GPU_ISOSURFACE
 //!A class to use GPU shaders to draw isosurfaces
@@ -711,6 +898,8 @@ private:
 	bool shadersOK;
 	//should we draw the thing in wireframe?
 	bool wireframe;
+	//Pointer to data
+	char *dataset;
 
 
 	struct HPMCConstants* hpmc_c;
@@ -739,8 +928,14 @@ private:
 	GLuint flat_v;
 	GLuint flat_p;
 
-	//Isosurface scalar threshold
+	//Isosurface scalar threshold (true value)
 	float threshold;
+
+	//true data maximum
+	float trueMax;
+
+	//Voxel data Bounding box
+	BoundCube bounds;
 	
 	//Compile shader for video card	
 	void compileShader( GLuint shader, const std::string& what );
@@ -752,13 +947,39 @@ public:
 	~DrawIsoSurfaceWithShader();
 
 	//initialise dataset and shaders
-	bool init(unsigned int dataX,
-		unsigned int dataY, unsigned int dataZ, const char *dataset);
+	bool init(const Voxels<float> &v);
 	//Draw
 	void draw() const;
 
-	void setScalarThresh(float thresh) { threshold=thresh;};
+	//Can the shader run?
+	bool canRun() const{return shadersOK;};
+
+	void setScalarThresh(float thresh) ;
+
+	void getBoundingBox(BoundCube &b) const;
 };
 #endif
 
+class DrawAxis : public DrawableObj
+{
+	private:
+		//!Drawing style
+		unsigned int style;
+		Point3D position;
+		//!size
+		float size;
+	public:
+		DrawAxis();
+		~DrawAxis();
+
+		//!Draw object
+		void draw() const;
+
+
+		void setStyle(unsigned int style);
+		void setSize(float newSize);
+		void setPosition(const Point3D &p);
+
+		void getBoundingBox(BoundCube &b) const;
+};
 #endif
