@@ -36,6 +36,12 @@
 //enable plot cutting and colouring
 #define MGL_GTE_1_10
 
+//!Plot error bar estimation strings
+const char *errModeStrings[] = { 
+				"None",
+				"Moving avg."
+				};
+
 
 using std::string;
 using std::pair;
@@ -85,11 +91,73 @@ unsigned int plotID(const std::string &plotString)
 	if(plotString == "Stem")
 		return PLOT_TYPE_STEM;
 	ASSERT(false);
-	return PLOT_TYPE_LINES;
+}
+
+unsigned int plotErrmodeID(const std::string &s)
+{
+	for(unsigned int ui=0;ui<PLOT_ERROR_ENDOFENUM; ui++)
+	{
+		if(s ==  errModeStrings[ui])
+			return ui;
+	}
+	ASSERT(false);
+}
+
+string plotErrmodeString(unsigned int plotID)
+{
+	return errModeStrings[plotID];
 }
 
 //===
 
+void genErrBars(const std::vector<float> &x, const std::vector<float> &y, 
+			std::vector<float> &errBars, const PLOT_ERROR &errMode)
+{
+	switch(errMode.mode)
+	{
+		case PLOT_ERROR_NONE:
+			return;	
+		case PLOT_ERROR_MOVING_AVERAGE:
+		{
+			ASSERT(errMode.movingAverageNum);
+			errBars.resize(y.size());
+			for(int ui=0;ui<y.size();ui++)
+			{
+				float mean;
+				mean=0.0f;
+
+				//Compute the local mean
+				for(int uj=0;uj<errMode.movingAverageNum;uj++)
+				{
+					int idx;
+					idx= std::max((int)ui+(int)uj-(int)errMode.movingAverageNum/2,0);
+					idx=std::min(idx,(int)(y.size()-1));
+					ASSERT(idx<y.size());
+					mean+=y[idx];
+				}
+
+				mean/=(float)errMode.movingAverageNum;
+
+				//Compute the local stddev
+				float stddev;
+				stddev=0;
+				for(int uj=0;uj<errMode.movingAverageNum;uj++)
+				{
+					int idx;
+					idx= std::max((int)ui+(int)uj-(int)errMode.movingAverageNum/2,0);
+					idx=std::min(idx,(int)(y.size()-1));
+					stddev+=(y[idx]-mean)*(y[idx]-mean);
+				}
+
+				stddev=sqrtf(stddev/(float)errMode.movingAverageNum);
+				errBars[ui]=stddev;
+			}
+			break;
+		}
+	}
+
+}
+
 void Multiplot::clear(bool preserveVisiblity)
 {
 
@@ -114,14 +182,15 @@ void Multiplot::clear(bool preserveVisiblity)
 
 	plottingData.clear();
 	regions.clear();
-	uniqueIDHandler.clear();
+	plotIDHandler.clear();
+	regionIDHandler.clear();
 	plotChanged=true;
 }
 
 void Multiplot::setStrings(unsigned int plotID, const std::string &x, 
 				const std::string &y, const std::string &t)
 {
-	unsigned int plotPos=uniqueIDHandler.getPos(plotID);
+	unsigned int plotPos=plotIDHandler.getPos(plotID);
 	plottingData[plotPos].xLabel = strToWStr(x);
 	plottingData[plotPos].yLabel = strToWStr(y);
 	
@@ -131,7 +200,7 @@ void Multiplot::setStrings(unsigned int plotID, const std::string &x,
 
 void Multiplot::setParentData(unsigned int plotID, const void *parentObj, unsigned int idx)
 {
-	unsigned int plotPos=uniqueIDHandler.getPos(plotID);
+	unsigned int plotPos=plotIDHandler.getPos(plotID);
 	plottingData[plotPos].parentObject= parentObj;
 	plottingData[plotPos].parentPlotIndex=idx;
 	
@@ -139,7 +208,7 @@ void Multiplot::setParentData(unsigned int plotID, const void *parentObj, unsign
 }
 
 unsigned int Multiplot::addPlot(const vector<float> &vX, const vector<float> &vY,
-					bool isLog)
+				const PLOT_ERROR &errMode,bool isLog)
 {
 	PlotData newPlotData ;
 
@@ -154,6 +223,8 @@ unsigned int Multiplot::addPlot(const vector<float> &vX, const vector<float> &vY
 	std::copy(vX.begin(),vX.end(),newDataRef.xValues.begin());
 	newDataRef.yValues.resize(vY.size());
 	std::copy(vY.begin(),vY.end(),newDataRef.yValues.begin());
+	genErrBars(newDataRef.xValues,newDataRef.yValues,newDataRef.errBars,errMode);
+
 
 	//Compute minima and maxima of plot data, and keep a copy of it
 	float maxThis=-std::numeric_limits<float>::max();
@@ -186,16 +257,16 @@ unsigned int Multiplot::addPlot(const vector<float> &vX, const vector<float> &vY
 	newDataRef.r=(0);newDataRef.g=(0);newDataRef.b=(1);
 
 	//assign a unique identifier to this plot, by which it can be referenced
-	unsigned int uniqueID = uniqueIDHandler.genId(plottingData.size()-1);
+	unsigned int uniqueID = plotIDHandler.genId(plottingData.size()-1);
 	plotChanged=true;
 	return uniqueID;
 }
 
 
-unsigned int Multiplot::addPlot(const vector<std::pair<float,float> > &v,
-					bool isLog)
+unsigned int Multiplot::addPlot(const vector<std::pair<float,float> > &v, 
+						const PLOT_ERROR &errMode, bool isLog)
 {
-	PlotData newPlotData ;
+	PlotData newPlotData;
 
 	plottingData.push_back(newPlotData);
 	PlotData &newDataRef=plottingData.back();
@@ -211,6 +282,7 @@ unsigned int Multiplot::addPlot(const vector<std::pair<float,float> > &v,
 		newDataRef.xValues[ui]=v[ui].first;
 		newDataRef.yValues[ui]=v[ui].second;
 	}
+	genErrBars(newDataRef.xValues,newDataRef.yValues,newDataRef.errBars,errMode);
 
 	//Compute minima and maxima of plot data, and keep a copy of it
 	float maxThis=-std::numeric_limits<float>::max();
@@ -226,10 +298,23 @@ unsigned int Multiplot::addPlot(const vector<std::pair<float,float> > &v,
 	
 	maxThis=-std::numeric_limits<float>::max();
 	minThis=std::numeric_limits<float>::max();
-	for(unsigned int ui=0;ui<v.size();ui++)
+	if(newDataRef.errBars.size())
 	{
-		minThis=std::min(minThis,v[ui].second);
-		maxThis=std::max(maxThis,v[ui].second);
+		ASSERT(newDataRef.errBars.size() == v.size());
+		for(unsigned int ui=0;ui<v.size();ui++)
+		{
+			minThis=std::min(minThis,v[ui].second-newDataRef.errBars[ui]);
+			maxThis=std::max(maxThis,v[ui].second+newDataRef.errBars[ui]);
+		}
+	}
+	else
+	{
+		for(unsigned int ui=0;ui<v.size();ui++)
+		{
+			minThis=std::min(minThis,v[ui].second);
+			maxThis=std::max(maxThis,v[ui].second);
+		}
+
 	}
 	newDataRef.minY=minThis;
 	newDataRef.maxY=1.10f*maxThis; //The 1.10 is because mathgl chops off data
@@ -244,7 +329,7 @@ unsigned int Multiplot::addPlot(const vector<std::pair<float,float> > &v,
 	newDataRef.r=(0);newDataRef.g=(0);newDataRef.b=(1);
 
 	//assign a unique identifier to this plot, by which it can be referenced
-	unsigned int uniqueID = uniqueIDHandler.genId(plottingData.size()-1);
+	unsigned int uniqueID = plotIDHandler.genId(plottingData.size()-1);
 	plotChanged=true;
 	return uniqueID;
 }
@@ -255,13 +340,13 @@ void Multiplot::setType(unsigned int plotUniqueID,unsigned int mode)
 {
 
 	ASSERT(mode<PLOT_TYPE_ENDOFENUM);
-	plottingData[uniqueIDHandler.getPos(plotUniqueID)].plotType=mode;
+	plottingData[plotIDHandler.getPos(plotUniqueID)].plotType=mode;
 	plotChanged=true;
 }
 
 void Multiplot::setColours(unsigned int plotUniqueID, float rN,float gN,float bN) 
 {
-	unsigned int plotPos=uniqueIDHandler.getPos(plotUniqueID);
+	unsigned int plotPos=plotIDHandler.getPos(plotUniqueID);
 	plottingData[plotPos].r=rN;
 	plottingData[plotPos].g=gN;
 	plottingData[plotPos].b=bN;
@@ -272,6 +357,8 @@ void Multiplot::setColours(unsigned int plotUniqueID, float rN,float gN,float bN
 void Multiplot::setBounds(float xMin, float xMax,
 			float yMin,float yMax)
 {
+	ASSERT(xMin<xMax);
+	ASSERT(yMin<=yMax);
 	xUserMin=xMin;
 	yUserMin=std::max(0.0f,yMin);
 	xUserMax=xMax;
@@ -281,6 +368,33 @@ void Multiplot::setBounds(float xMin, float xMax,
 	plotChanged=true;
 }
 
+
+void Multiplot::disableUserAxisBounds(bool xBound)
+{
+	float xMin,xMax,yMin,yMax;
+	scanBounds(xMin,xMax,yMin,yMax);
+
+	if(xBound)
+	{
+		xUserMin=xMin;
+		xUserMax=xMax;
+	}
+	else
+	{
+		yUserMin=std::max(0.0f,yMin);
+		yUserMax=yMax;
+	}
+
+
+	//Check to see if we have zoomed all the bounds out anyway
+	if(fabs(xUserMin -xMin)<=std::numeric_limits<float>::epsilon() &&
+		fabs(yUserMin -yMin)<=std::numeric_limits<float>::epsilon())
+	{
+		applyUserBounds=false;
+	}
+
+	plotChanged=true;
+}
 void Multiplot::getBounds(float &xMin, float &xMax,
 			float &yMin,float &yMax) const
 {
@@ -288,18 +402,28 @@ void Multiplot::getBounds(float &xMin, float &xMax,
 	{
 		xMin=xUserMin;
 		yMin=yUserMin;
-		xMin=xUserMax;
-		yMin=yUserMax;
+		xMax=xUserMax;
+		yMax=yUserMax;
 	}
 	else
 	{
-		//OK, we are going to have to scan for max/min
-		xMin=std::numeric_limits<float>::max();
-		xMax=-std::numeric_limits<float>::max();
-		yMin=std::numeric_limits<float>::max();
-		yMax=-std::numeric_limits<float>::max();
+		scanBounds(xMin,xMax,yMin,yMax);
+	}
 
-		for(unsigned int ui=0;ui<plottingData.size();ui++)
+	ASSERT(xMin <xMax && yMin <=yMax);
+}
+
+void Multiplot::scanBounds(float &xMin,float &xMax,float &yMin,float &yMax) const
+{
+	//OK, we are going to have to scan for max/min
+	xMin=std::numeric_limits<float>::max();
+	xMax=-std::numeric_limits<float>::max();
+	yMin=std::numeric_limits<float>::max();
+	yMax=-std::numeric_limits<float>::max();
+
+	for(unsigned int ui=0;ui<plottingData.size();ui++)
+	{
+		if(plottingData[ui].visible)
 		{
 			xMin=std::min(plottingData[ui].minX,xMin);
 			xMax=std::max(plottingData[ui].maxX,xMax);
@@ -307,7 +431,14 @@ void Multiplot::getBounds(float &xMin, float &xMax,
 			yMin=std::min(plottingData[ui].minY,yMin);
 			yMax=std::max(plottingData[ui].maxY,yMax);
 		}
+	}
 
+	//If we are in log mode, then we need to set the
+	//log of that bound before emitting it.
+	if(isLogarithmic() && yMax)
+	{
+		yMin=log10(std::max(yMin,1.0f));
+		yMax=log10(yMax);
 	}
 }
 
@@ -337,6 +468,21 @@ void Multiplot::bestEffortRestoreVisibility()
 	plotChanged=true;
 }
 
+//Is the plot being displayed in lgo mode?
+bool Multiplot::isLogarithmic() const
+{
+	for(unsigned int ui=0;ui<plottingData.size(); ui++)
+	{
+		if(plottingData[ui].visible)
+		{
+			if(plottingData[ui].logarithmic) 
+				return true;
+		}
+	}
+
+	return false;
+}
+
 void Multiplot::drawPlot(mglGraph *gr) const
 {
 	if(!plottingData.size())
@@ -351,6 +497,7 @@ void Multiplot::drawPlot(mglGraph *gr) const
 	maxX=-std::numeric_limits<float>::max();
 	minY=std::numeric_limits<float>::max();
 	maxY=-std::numeric_limits<float>::max();
+
 	std::wstring xLabel,yLabel,plotTitle;
 	bool notLog=false;
 	for(unsigned int ui=0;ui<plottingData.size(); ui++)
@@ -409,8 +556,8 @@ void Multiplot::drawPlot(mglGraph *gr) const
 	mglPoint min,max;
 	if(applyUserBounds)
 	{
-		ASSERT(yUserMax > yUserMin);
-		ASSERT(xUserMax > xUserMin);
+		ASSERT(yUserMax >=yUserMin);
+		ASSERT(xUserMax >=xUserMin);
 
 		max.x =xUserMax;
 		max.y=yUserMax;
@@ -440,6 +587,20 @@ void Multiplot::drawPlot(mglGraph *gr) const
 		
 		gr->Org=axisCross;
 	}
+
+
+	//"Push" bounds around to prevent min == max
+	if(min.x == max.x)
+	{
+		min.x-=0.5;
+		max.x+=0.5;
+	}
+
+	if(min.y == max.y)
+	{
+		max.y+=1;
+	}
+
 	gr->Axis(min,max,axisCross);
 
 	//I don't like the default, nor this. 
@@ -465,7 +626,7 @@ void Multiplot::drawPlot(mglGraph *gr) const
 		for(unsigned int uj=0;uj<regions.size();uj++)
 		{
 			//Do not plot regions that do not belong to current plot
-			if(regions[uj].ownerPlot!= uniqueIDHandler.getId(ui))
+			if(regions[uj].ownerPlot!= plotIDHandler.getId(ui))
 				continue;
 			//Compute region bounds, such that it will not exceed the axis
 			float rMinX, rMaxX, rMinY,rMaxY;
@@ -481,7 +642,7 @@ void Multiplot::drawPlot(mglGraph *gr) const
 				gr->FaceZ(rMinX,rMinY,-1,rMaxX-rMinX,rMaxY-rMinY,
 						colourCode);
 						
-				regionNum+1;
+				regionNum++;
 			}
 		}
 	}
@@ -493,15 +654,21 @@ void Multiplot::drawPlot(mglGraph *gr) const
 	unsigned int plotNum=0;
 	for(unsigned int ui=0;ui<plottingData.size(); ui++)
 	{
+		bool showErrs;
 
-		mglData xDat,yDat;
+		mglData xDat,yDat,eDat;
 		if(!plottingData[ui].visible)
 			continue;	
 		
-		float *bufferX,*bufferY;
+		float *bufferX,*bufferY,*bufferErr;
 		bufferX = new float[plottingData[ui].xValues.size()];
 		bufferY = new float[plottingData[ui].yValues.size()];
-		
+
+		showErrs=plottingData[ui].errBars.size();
+		bufferErr = new float[plottingData[ui].errBars.size()];
+
+
+
 		if(logarithmic)
 		{
 			for(unsigned int uj=0;uj<plottingData[ui].xValues.size(); uj++)
@@ -521,11 +688,22 @@ void Multiplot::drawPlot(mglGraph *gr) const
 			{
 				bufferX[uj] = plottingData[ui].xValues[uj];
 				bufferY[uj] = plottingData[ui].yValues[uj];
+
+			}
+			if(showErrs)
+			{
+				for(unsigned int uj=0;uj<plottingData[ui].errBars.size(); uj++)
+					bufferErr[uj] = plottingData[ui].errBars[uj];
 			}
 		}
+	
+		//Mathgl needs to know where to put the error bars.	
+		ASSERT(!showErrs  || plottingData[ui].errBars.size() ==plottingData[ui].xValues.size());
 		
 		xDat.Set(bufferX,plottingData[ui].xValues.size());
 		yDat.Set(bufferY,plottingData[ui].yValues.size());
+	
+		eDat.Set(bufferErr,plottingData[ui].errBars.size());
 
 		//Fake it.
 		colourCode[0]= getNearestMathglColour(plottingData[ui].r,
@@ -542,6 +720,8 @@ void Multiplot::drawPlot(mglGraph *gr) const
 				gr->SetCut(true);
 				
 				gr->Plot(xDat,yDat,colourCode);
+				if(showErrs)
+					gr->Error(xDat,yDat,eDat,colourCode);
 				gr->SetCut(false);
 #else
 				gr->Plot(xDat,yDat);
@@ -579,10 +759,12 @@ void Multiplot::drawPlot(mglGraph *gr) const
 				break;
 		}
 
-		gr->AddLegend(plottingData[ui].title.c_str(),colourCode);
+		if(drawLegend)
+			gr->AddLegend(plottingData[ui].title.c_str(),colourCode);
 
 		delete[]  bufferX;
 		delete[]  bufferY;
+		delete[]  bufferErr;
 
 		plotNum++;
 	}
@@ -594,7 +776,7 @@ void Multiplot::drawPlot(mglGraph *gr) const
 	string sY;
 	sY.assign(yLabel.begin(), yLabel.end()); //unicode conversion
 	if(logarithmic && !notLog)
-		sY = string("\\log(") + sY + ")";
+		sY = string("\\log_{10}(") + sY + ")";
 	else if (logarithmic && notLog)
 	{
 		sY = string("Mixed log/non-log:") + sY ;
@@ -629,14 +811,14 @@ void Multiplot::hideAll()
 
 void Multiplot::setVisible(unsigned int uniqueID, bool setVis)
 {
-	unsigned int plotPos = uniqueIDHandler.getPos(uniqueID);
+	unsigned int plotPos = plotIDHandler.getPos(uniqueID);
 
 	plottingData[plotPos].visible=setVis;
 	plotChanged=true;
 }
 
-void Multiplot::addRegion(unsigned int parentPlot,float start, float end, 
-			float rNew, float gNew, float bNew)
+void Multiplot::addRegion(unsigned int parentPlot,unsigned int regionID,float start, float end, 
+			float rNew, float gNew, float bNew, Filter *parentFilter)
 {
 	ASSERT(start <end);
 	ASSERT( rNew>=0.0 && rNew <= 1.0);
@@ -646,6 +828,9 @@ void Multiplot::addRegion(unsigned int parentPlot,float start, float end,
 	PlotRegion region;
 	region.ownerPlot=parentPlot;
 	region.bounds=std::make_pair(start,end);
+	region.parentFilter = parentFilter;
+	region.id = regionID;
+	region.uniqueID = regionIDHandler.genId(regions.size());
 
 	region.r=rNew;
 	region.g=gNew;
@@ -670,34 +855,66 @@ unsigned int Multiplot::getNumVisible() const
 
 bool Multiplot::isPlotVisible(unsigned int plotID) const
 {
-	return plottingData[uniqueIDHandler.getPos(plotID)].visible;
+	return plottingData[plotIDHandler.getPos(plotID)].visible;
 }
 
-void Multiplot::getRawData(std::vector<std::vector<std::pair< float,float> > > &rawData,
-				std::vector<std::pair<std::wstring,std::wstring> > &labels) const
+void Multiplot::getRawData(std::vector<std::vector<std::vector< float> > > &rawData,
+				std::vector<std::vector<std::wstring> > &labels) const
 {
-	std::vector<pair<float,float> > tmp;
+	std::vector<vector<float> > tmp;
 	labels.clear();
 	for(unsigned int ui=0; ui<plottingData.size();ui++)
 	{
 		if(plottingData[ui].visible)
 		{
+			vector<float> datX;
+			vector<float> datY;
+			vector<float> datE;
 
+			std::vector<std::wstring> strVec;
+			
 			for(unsigned int uj=0;uj<plottingData[ui].xValues.size(); uj++)
 			{
-				tmp.push_back(std::make_pair(plottingData[ui].xValues[uj],
-							plottingData[ui].yValues[uj]));
+				datX.push_back(plottingData[ui].xValues[uj]);
+				datY.push_back(plottingData[ui].yValues[uj]);
+				if(plottingData[ui].errBars.size())
+					datE.push_back(plottingData[ui].errBars[uj]);
+			}
+
+			if(datX.size())
+			{
+				tmp.push_back(datX);
+				tmp.push_back(datY);
+				strVec.push_back(plottingData[ui].xLabel);
+				strVec.push_back(plottingData[ui].title);
+			}
+
+			if(datE.size())
+			{
+				tmp.push_back(datE);
+				strVec.push_back(std::wstring(L"Error (std. dev)"));
 			}
 
-			labels.push_back(make_pair(plottingData[ui].xLabel,
-							plottingData[ui].title));
 
 			rawData.push_back(tmp);
+			labels.push_back(strVec);
 			tmp.clear();
 		}
 	}
 }
 
+void Multiplot::getRegions(std::vector<PlotRegion> &copyRegions) const
+{
+	
+	copyRegions.reserve(regions.size());
+
+	for(unsigned int ui=0;ui<regions.size();ui++)
+	{
+		if(isPlotVisible(regions[ui].ownerPlot))
+			copyRegions.push_back(regions[ui]); 
+	}
+}
+
 
 char Multiplot::getNearestMathglColour(float r, float g, float b) const
 {
@@ -717,7 +934,8 @@ char Multiplot::getNearestMathglColour(float r, float g, float b) const
 	//mathgl's "named" colours
 	const char *mglColourString="wkrgbcymhRGBCYMHWlenuqpLENUQP";
 	const unsigned int numColours= strlen(mglColourString);
-	
+
+	ASSERT(numColours);	
 	mglColor c;
 	char val;
 
@@ -750,3 +968,120 @@ char Multiplot::getNearestMathglColour(float r, float g, float b) const
 
 	return val;
 }
+
+
+float Multiplot::moveRegionTest(unsigned int regionID, 
+			unsigned int method, float newPos)  const
+{
+	unsigned int region=(unsigned int)-1;
+	for(unsigned int ui=0;ui<regions.size();ui++)
+	{
+		if(regionID== regions[ui].uniqueID)
+			region=ui;
+	}
+
+	ASSERT(region<regions.size());
+	ASSERT(isPlotVisible(regions[region].ownerPlot));
+
+	//Check that moving this range will not cause any overlaps with 
+	//other regions
+	float mean;
+	mean=(regions[region].bounds.first + regions[region].bounds.second)/2.0f;
+
+	//Who is the owner of the current plot -- we only want to interact with our colleagues
+	unsigned int curOwnerPlot= regions[region].ownerPlot;
+	float xMin,xMax,yMin,yMax;
+	getBounds(xMin,xMax,yMin,yMax);
+
+	switch(method)
+	{
+		//Left extend
+		case REGION_LEFT_EXTEND:
+			//Check that the upper bound does not intersect any RHS of 
+			//region bounds
+			for(unsigned int ui=0; ui<regions.size(); ui++)
+			{
+				if(regions[ui].ownerPlot == curOwnerPlot &&
+					(regions[ui].bounds.second < mean && ui !=region) )
+						newPos=std::max(newPos,regions[ui].bounds.second);
+			}
+			//Dont allow past self right
+			newPos=std::min(newPos,regions[region].bounds.second);
+			//Dont extend outside plot
+			newPos=std::max(newPos,xMin);
+			break;
+		//shift
+		case REGION_MOVE:
+			//Check that the upper bound does not intersect any RHS or LHS of 
+			//region bounds
+			if(newPos > mean) 
+				
+			{
+				//Disallow hitting other bounds
+				for(unsigned int ui=0; ui<regions.size(); ui++)
+				{
+					if(regions[ui].ownerPlot == curOwnerPlot &&
+						(regions[ui].bounds.first > mean && ui != region) )
+						newPos=std::min(newPos,regions[ui].bounds.first);
+				}
+				newPos=std::max(newPos,xMin);
+			}
+			else
+			{
+				//Disallow hitting other bounds
+				for(unsigned int ui=0; ui<regions.size(); ui++)
+				{
+					if(regions[ui].ownerPlot == curOwnerPlot &&
+						(regions[ui].bounds.second < mean && ui != region))
+						newPos=std::max(newPos,regions[ui].bounds.second);
+				}
+				//Dont extend outside plot
+				newPos=std::min(newPos,xMax);
+			}
+			break;
+		//Right extend
+		case REGION_RIGHT_EXTEND:
+			//Disallow hitting other bounds
+
+			for(unsigned int ui=0; ui<regions.size(); ui++)
+			{
+				if(regions[ui].ownerPlot == curOwnerPlot &&
+					(regions[ui].bounds.second > mean && ui != region))
+					newPos=std::min(newPos,regions[ui].bounds.first);
+			}
+			//Dont allow past self left
+			newPos=std::max(newPos,regions[region].bounds.first);
+			//Dont extend outside plot
+			newPos=std::min(newPos,xMax);
+			break;
+		default:
+			ASSERT(false);
+	}
+
+
+
+	return newPos;
+}
+
+
+void Multiplot::moveRegion(unsigned int regionID, unsigned int method, float newPos)
+{
+
+	//Well, we should have called this externally to determine location
+	//let us confirm that this is the case 
+	moveRegionTest(regionID,method,newPos);
+
+	unsigned int region=(unsigned int)-1;
+	for(unsigned int ui=0;ui<regions.size();ui++)
+	{
+		if(regionID== regions[ui].uniqueID)
+			region=ui;
+	}
+
+	ASSERT(region<regions.size());
+	ASSERT(regions[region].parentFilter);
+	regions[region].parentFilter->setPropFromRegion(method,regions[region].id,newPos);	
+
+
+
+}
