@@ -30,6 +30,11 @@ Scene::Scene() : tempCam(0), activeCam(0), cameraSet(false), outWinAspect(1.0f),
 	viewRestrict=false;
 	useAlpha=true;
 	useLighting=true;
+	useEffects=false;
+	showAxis=true;
+
+	//default to black
+	rBack=gBack=bBack=0.0f;
 
 }
 
@@ -38,7 +43,7 @@ Scene::~Scene()
 	clearAll();
 }
 
-void Scene::draw() 
+unsigned int Scene::initDraw()
 {
 	glClear(GL_COLOR_BUFFER_BIT |GL_DEPTH_BUFFER_BIT);
 
@@ -61,10 +66,8 @@ void Scene::draw()
 	glLightfv(GL_LIGHT0, GL_POSITION, light_position);
 
 
-	applyGLExtentions();
 
 
-	bool lightsOn=false;
 	glDisable(GL_LIGHTING);
 	//==
 	
@@ -74,246 +77,401 @@ void Scene::draw()
 		if(!boundCube.isValid())
 			computeSceneLimits();
 
-		glPushMatrix();
-		Point3D lightNormal;
-		if(tempCam)
-		{
-			//Inform text about current camera, so it can billboard if needed
-			DrawGLText::setCurCamera(tempCam);
-
-			tempCam->apply(outWinAspect,boundCube);
-			lightNormal= tempCam->getViewDirection();
-
-		}
-		else
-		{
-			ASSERT(activeCam < cameras.size());
-			//Inform text about current camera, so it can billboard if needed
-			DrawGLText::setCurCamera(cameras[activeCam]);
-
-			//If viewport restriction is on, inform camera to
-			//shrink viewport to specified region
-			if(viewRestrict)
-			{
-				cameras[activeCam]->apply(outWinAspect,boundCube,true,
-						viewRestrictStart[0],viewRestrictEnd[0],
-						viewRestrictStart[1],viewRestrictEnd[1]);
-			}
-			else
-				cameras[activeCam]->apply(outWinAspect,boundCube);
-
-			lightNormal= cameras[activeCam]->getViewDirection();
-		}
-
-
-
-		glNormal3f(lightNormal[0],lightNormal[1],lightNormal[2]);	
 	}
 
 
 
-	//TODO: Alpha blending is, as always, causing problems
-	// we need to do a quick and dirty and draw in opaque objects
-	// then depth sort the alpha'd objects as best we can, then give up.
-	// depth buffering may need to be toggled during alpha'd object draw.
-	// For now, avoid alpha blended surface represented objects.
+	//Let the effects objects know about the scene
+	Effect::setBoundingCube(boundCube);
 
-	//Draw the referenced objecst
-	for(unsigned int ui=0; ui<refObjects.size(); ui++)
+	unsigned int passes=1;
+
+	if(useEffects)
 	{
-		if(useLighting)
-		{
-			if(!refObjects[ui]->wantsLight && lightsOn)
-			{
-				lightsOn=false;
-				glDisable(GL_LIGHTING);
-			}
-			else if (refObjects[ui]->wantsLight && !lightsOn)
-			{
-				glEnable(GL_LIGHTING);
-				lightsOn=true;
-			}
-		}
-
-		refObjects[ui]->draw();	
+		for(unsigned int ui=0;ui<effects.size();ui++)
+			passes=std::max(passes,effects[ui]->numPassesNeeded());
 	}
 
-	//Set up the objects
-	for(unsigned int ui=0; ui<objects.size(); ui++)
-	{
-		if(useLighting)
-		{	
-			if(!objects[ui]->wantsLight && lightsOn )
-			{
-				//Object prefers doing its thing in the dark
-				glDisable(GL_LIGHTING);
-				lightsOn=false;
-			}
-			else if (objects[ui]->wantsLight && !lightsOn)
-			{
-				glEnable(GL_LIGHTING);
-				lightsOn=true;
-			}
-		}
-
-		//If we are in selection mode, draw the bounding box
-		//if the object is selected.
-		if(ui == lastSelected && selectionMode)
-		{
-			//May be required for selection box drawing
-			BoundCube bObject;
-			DrawRectPrism p;
-			//Get the bounding box for the object & draw it
-			objects[ui]->getBoundingBox(bObject);
-			p.setAxisAligned(bObject);
-			p.setColour(0,0.2,1,0.5); //blue-greenish
-			p.draw();
-
-		}
-		else if( ui == lastHovered)
-		{	
-			//Search for a binding
-			bool haveBinding;
-			haveBinding=false;
-			
-			vector<const SelectionBinding *> binder;
-			for(unsigned int uj=0;uj<selectionDevices.size();uj++)
-			{
-				if(selectionDevices[uj]->getAvailBindings(objects[lastHovered],binder))
-				{
-					haveBinding=true;
-					break;
-				}
-			}
-			
-
-			if(haveBinding)
-			{	
-				glPushAttrib(GL_LIGHTING);
-				glDisable(GL_LIGHTING);
-
-				//Now draw some hints for the binding itself as a 2D overlay
-				//
-				//Draw the action type (translation, rotation etc)
-				//and the button it is bound to
-				DrawTexturedQuadOverlay binderIcons,mouseIcons,keyIcons;
-
-
-				const float ICON_SIZE= 0.05;
-				binderIcons.setTexturePool(&texPool);
-				binderIcons.setWindowSize(winX,winY);	
-				binderIcons.setSize(ICON_SIZE*winY);
-				
-				mouseIcons.setTexturePool(&texPool);
-				mouseIcons.setWindowSize(winX,winY);	
-				mouseIcons.setSize(ICON_SIZE*winY);
-			
-				keyIcons.setTexturePool(&texPool);
-				keyIcons.setWindowSize(winX,winY);	
-				keyIcons.setSize(ICON_SIZE*winY);
-
-				unsigned int iconNum=0;
-				for(unsigned int ui=0;ui<binder.size();ui++)
-				{
-					bool foundIconTex,foundMouseTex;
-					foundIconTex=false;
-					foundMouseTex=false;
-					switch(binder[ui]->getInteractionMode())
-					{
-						case BIND_MODE_FLOAT_SCALE:
-						case BIND_MODE_FLOAT_TRANSLATE:
-							foundIconTex=binderIcons.setTexture(TEST_OVERLAY_PNG[TEXTURE_ENLARGE]);
-							break;
-						case BIND_MODE_POINT3D_TRANSLATE:
-							foundIconTex=binderIcons.setTexture(TEST_OVERLAY_PNG[TEXTURE_TRANSLATE]);
-							break;
-						case BIND_MODE_POINT3D_ROTATE:
-							foundIconTex=binderIcons.setTexture(TEST_OVERLAY_PNG[TEXTURE_ROTATE]);
-						default:
-							break;
-					}
-
-					//Draw the mouse action
-					switch(binder[ui]->getMouseButtons())
-					{
-						case SELECT_BUTTON_LEFT:
-							foundMouseTex=mouseIcons.setTexture(TEST_OVERLAY_PNG[TEXTURE_LEFT_CLICK]);
-							break;
-						case SELECT_BUTTON_MIDDLE:
-							foundMouseTex=mouseIcons.setTexture(TEST_OVERLAY_PNG[TEXTURE_MIDDLE_CLICK]);
-							break;
-						case SELECT_BUTTON_RIGHT:
-							foundMouseTex=mouseIcons.setTexture(TEST_OVERLAY_PNG[TEXTURE_RIGHT_CLICK]);
-							break;
-						default:
-							//The flags are or'd together, so we can get other combinations
-							break;
-					}
-
-					bool foundKeyTex;
-					foundKeyTex=false;
-					//Draw the keyboard action, if any
-					switch(binder[ui]->getKeyFlags())
-					{
-						case FLAG_CMD:
-#ifdef __APPLE__
-							foundKeyTex=keyIcons.setTexture(TEST_OVERLAY_PNG[TEXTURE_COMMAND]);
-#else
-							foundKeyTex=keyIcons.setTexture(TEST_OVERLAY_PNG[TEXTURE_CTRL]);
-#endif
-							break;
-						case FLAG_SHIFT:
-							foundKeyTex=keyIcons.setTexture(TEST_OVERLAY_PNG[TEXTURE_SHIFT]);
-							break;
-						default:
-							//The flags are or'd together, so we can get other combinations
-							break;
-					}
-
-
-					if(foundIconTex && foundMouseTex )
-					{
-						const float SPACING=0.75*ICON_SIZE;
-						if(foundKeyTex)
-						{
-							//Make room for keyTex
-							binderIcons.setPos((0.93+SPACING)*winX,ICON_SIZE*winY*(1+(float)iconNum));
-							keyIcons.setPos(0.93*winX,ICON_SIZE*winY*(1+(float)iconNum));
-							mouseIcons.setPos((0.93-SPACING)*winX,ICON_SIZE*winY*(1+(float)iconNum));
-						}
-						else
-						{
-							binderIcons.setPos(0.95*winX,ICON_SIZE*winY*(1+(float)iconNum));
-							mouseIcons.setPos(0.90*winX,ICON_SIZE*winY*(1+(float)iconNum));
-						}
-
-						binderIcons.draw();
-						mouseIcons.draw();
-
-						if(foundKeyTex)
-							keyIcons.draw();
-
-						iconNum++;
-					}
-					
-				}
-
-				glPopAttrib();
-
-			}
-		}
+	return passes;
+}
 
+void Scene::updateCam(const Camera *camToUse) const
+{
+	Point3D lightNormal;
 	
 
-		objects[ui]->draw();
+	glLoadIdentity();
+	//If viewport restriction is on, inform camera to
+	//shrink viewport to specified region
+	if(viewRestrict)
+	{
+		camToUse->apply(outWinAspect,boundCube,true,
+				viewRestrictStart[0],viewRestrictEnd[0],
+				viewRestrictStart[1],viewRestrictEnd[1]);
+	}
+	else
+		camToUse->apply(outWinAspect,boundCube);
+
+	lightNormal=camToUse->getViewDirection();
+	glNormal3f(lightNormal[0],lightNormal[1],lightNormal[2]);	
+
+}
+
+void Scene::draw() 
+{
+
+	glPushMatrix();
+
+
+	Camera *camToUse;
+	if(tempCam)
+		camToUse=tempCam;
+	else
+	{
+		ASSERT(activeCam < cameras.size());
+		camToUse=cameras[activeCam];
+	}
+	//Inform text about current camera, so it can billboard if needed
+	DrawGLText::setCurCamera(camToUse);
+	DrawField3D::setCurCamera(camToUse);
+	
+	Effect::setCurCam(camToUse);
+
+
+	bool lightsOn=false;
+	unsigned int numberTotalPasses;
+	numberTotalPasses=initDraw();
+
+	unsigned int passNumber=0;
+
+	if(cameraSet)
+		updateCam(camToUse);
+
+
+
+	bool needCamUpdate=false;
+	while(passNumber < numberTotalPasses)
+	{
+
+		if(useEffects)
+		{
+			for(unsigned int ui=0;ui<effects.size();ui++)
+			{
+				effects[ui]->enable(passNumber);
+				needCamUpdate|=effects[ui]->needCamUpdate();
+			}
+
+			if(cameraSet && needCamUpdate)
+			{
+				glLoadIdentity();	
+				updateCam(camToUse);
+			}
+		}
+
+
+		if(showAxis)
+		{
+			if(useLighting)
+				glEnable(GL_LIGHTING);
+			DrawAxis a;
+			a.setStyle(AXIS_IN_SPACE);
+			a.setSize(boundCube.getLargestDim());
+			a.setPosition(boundCube.getCentroid());
+
+			a.draw();
+			if(useLighting)
+				glDisable(GL_LIGHTING);
+		}
+		//Draw the referenced objects
+		for(unsigned int ui=0; ui<refObjects.size(); ui++)
+		{
+			//overlays need to be drawn later
+			if(refObjects[ui]->isOverlay())
+				continue;
+
+			if(useLighting)
+			{
+				if(!refObjects[ui]->wantsLight && lightsOn)
+				{
+					lightsOn=false;
+					glDisable(GL_LIGHTING);
+				}
+				else if (refObjects[ui]->wantsLight && !lightsOn)
+				{
+					glEnable(GL_LIGHTING);
+					lightsOn=true;
+				}
+			}
+
+			refObjects[ui]->draw();	
+		}
+
+		//Set up the objects
+		for(unsigned int ui=0; ui<objects.size(); ui++)
+		{
+			//overlays need to be drawn later
+			if(objects[ui]->isOverlay())
+				continue;
+			if(useLighting)
+			{	
+				if(!objects[ui]->wantsLight && lightsOn )
+				{
+					//Object prefers doing its thing in the dark
+					glDisable(GL_LIGHTING);
+					lightsOn=false;
+				}
+				else if (objects[ui]->wantsLight && !lightsOn)
+				{
+					glEnable(GL_LIGHTING);
+					lightsOn=true;
+				}
+			}
+
+			//If we are in selection mode, draw the bounding box
+			//if the object is selected.
+			if(ui == lastSelected && selectionMode)
+			{
+				//May be required for selection box drawing
+				BoundCube bObject;
+				DrawRectPrism p;
+				//Get the bounding box for the object & draw it
+				objects[ui]->getBoundingBox(bObject);
+				p.setAxisAligned(bObject);
+				p.setColour(0,0.2,1,0.5); //blue-greenish
+				if(lightsOn)
+					glDisable(GL_LIGHTING);
+				p.draw();
+				if(lightsOn)
+					glEnable(GL_LIGHTING);
+
+			}
+
+			objects[ui]->draw();
+		}
+		
+
+		glFlush();
+		passNumber++;
+	}
+	
+	
+	//Disable effects
+	if(useEffects)
+	{
+		//Disable them in reverse order to simulate a stack-type
+		//behaviour.
+		for(unsigned int ui=effects.size();ui!=0;)
+		{
+			ui--;
+			effects[ui]->disable();
+		}
+		//glPopMatrix();
 	}
 
 
-	if(cameraSet)
-		glPopMatrix();
+	glPopMatrix();
+		
+	//Now draw 2D overlays
+	if(lastHovered != (unsigned int)(-1))
+		drawHoverOverlay();
+	drawOverlays();
 
 }
 
+void Scene::drawOverlays() const
+{
+
+	//Custom projection matrix
+	glMatrixMode(GL_PROJECTION);
+	glPushMatrix();
+	glLoadIdentity();
+	//Set the opengl camera state back into modelview mode
+	if(viewRestrict)
+	{
+
+		//FIXME: How does the aspect ratio fit in here?
+		gluOrtho2D(viewRestrictStart[0],
+				viewRestrictEnd[0],
+				viewRestrictStart[0],
+				viewRestrictStart[1]);
+	}
+	else
+		gluOrtho2D(0, outWinAspect, 1.0, 0);
+
+
+
+	glMatrixMode(GL_MODELVIEW);
+	glPushMatrix();
+	glLoadIdentity();
+
+	glDisable(GL_DEPTH_TEST);
+
+	for(unsigned int ui=0;ui<refObjects.size();ui++)
+	{
+		if(refObjects[ui]->isOverlay())
+			refObjects[ui]->draw();
+	}
+	
+	for(unsigned int ui=0;ui<objects.size();ui++)
+	{
+		if(objects[ui]->isOverlay())
+			objects[ui]->draw();
+	}
+	
+
+	glEnable(GL_DEPTH_TEST);
+	glPopMatrix();
+	glMatrixMode(GL_PROJECTION);
+	glPopMatrix();
+
+	glMatrixMode(GL_MODELVIEW);
+}
+
+void Scene::drawHoverOverlay()
+{
+
+	glEnable(GL_ALPHA_TEST);
+	glDisable(GL_DEPTH_TEST);
+	//Search for a binding
+	bool haveBinding;
+	haveBinding=false;
+
+	//Prevent transparent areas from interactiing
+	//with the depth buffer
+	glAlphaFunc(GL_GREATER,0.01f);
+	
+	vector<const SelectionBinding *> binder;
+	for(unsigned int uj=0;uj<selectionDevices.size();uj++)
+	{
+		if(selectionDevices[uj]->getAvailBindings(objects[lastHovered],binder))
+		{
+			haveBinding=true;
+			break;
+		}
+	}
+
+
+	if(haveBinding)
+	{	
+		glPushAttrib(GL_LIGHTING);
+		glDisable(GL_LIGHTING);
+
+		//Now draw some hints for the binding itself as a 2D overlay
+		//
+		//Draw the action type (translation, rotation etc)
+		//and the button it is bound to
+		DrawTexturedQuadOverlay binderIcons,mouseIcons,keyIcons;
+
+
+		const float ICON_SIZE= 0.05;
+		binderIcons.setTexturePool(&texPool);
+		binderIcons.setWindowSize(winX,winY);	
+		binderIcons.setSize(ICON_SIZE*winY);
+		
+		mouseIcons.setTexturePool(&texPool);
+		mouseIcons.setWindowSize(winX,winY);	
+		mouseIcons.setSize(ICON_SIZE*winY);
+	
+		keyIcons.setTexturePool(&texPool);
+		keyIcons.setWindowSize(winX,winY);	
+		keyIcons.setSize(ICON_SIZE*winY);
+
+		unsigned int iconNum=0;
+		for(unsigned int ui=0;ui<binder.size();ui++)
+		{
+			bool foundIconTex,foundMouseTex;
+			foundIconTex=false;
+			foundMouseTex=false;
+			switch(binder[ui]->getInteractionMode())
+			{
+				case BIND_MODE_FLOAT_SCALE:
+				case BIND_MODE_FLOAT_TRANSLATE:
+				case BIND_MODE_POINT3D_SCALE:
+					foundIconTex=binderIcons.setTexture(TEST_OVERLAY_PNG[TEXTURE_ENLARGE]);
+					break;
+				case BIND_MODE_POINT3D_TRANSLATE:
+					foundIconTex=binderIcons.setTexture(TEST_OVERLAY_PNG[TEXTURE_TRANSLATE]);
+					break;
+				case BIND_MODE_POINT3D_ROTATE:
+				case BIND_MODE_POINT3D_ROTATE_LOCK:
+					foundIconTex=binderIcons.setTexture(TEST_OVERLAY_PNG[TEXTURE_ROTATE]);
+				default:
+					break;
+			}
+
+			//Draw the mouse action
+			switch(binder[ui]->getMouseButtons())
+			{
+				case SELECT_BUTTON_LEFT:
+					foundMouseTex=mouseIcons.setTexture(TEST_OVERLAY_PNG[TEXTURE_LEFT_CLICK]);
+					break;
+				case SELECT_BUTTON_MIDDLE:
+					foundMouseTex=mouseIcons.setTexture(TEST_OVERLAY_PNG[TEXTURE_MIDDLE_CLICK]);
+					break;
+				case SELECT_BUTTON_RIGHT:
+					foundMouseTex=mouseIcons.setTexture(TEST_OVERLAY_PNG[TEXTURE_RIGHT_CLICK]);
+					break;
+				default:
+					//The flags are or'd together, so we can get other combinations
+					break;
+			}
+
+			bool foundKeyTex;
+			foundKeyTex=false;
+			//Draw the keyboard action, if any
+			switch(binder[ui]->getKeyFlags())
+			{
+				case FLAG_CMD:
+#ifdef __APPLE__
+					foundKeyTex=keyIcons.setTexture(TEST_OVERLAY_PNG[TEXTURE_COMMAND]);
+#else
+					foundKeyTex=keyIcons.setTexture(TEST_OVERLAY_PNG[TEXTURE_CTRL]);
+#endif
+					break;
+				case FLAG_SHIFT:
+					foundKeyTex=keyIcons.setTexture(TEST_OVERLAY_PNG[TEXTURE_SHIFT]);
+					break;
+				default:
+					//The flags are or'd together, so we can get other combinations
+					break;
+			}
+
+			if(foundIconTex && foundMouseTex )
+			{
+				const float SPACING=0.75*ICON_SIZE;
+				if(foundKeyTex)
+				{
+					//Make room for keyTex
+					binderIcons.setPos((0.93+SPACING)*winX,ICON_SIZE*winY*(1+(float)iconNum));
+					keyIcons.setPos(0.93*winX,ICON_SIZE*winY*(1+(float)iconNum));
+					mouseIcons.setPos((0.93-SPACING)*winX,ICON_SIZE*winY*(1+(float)iconNum));
+				}
+				else
+				{
+					binderIcons.setPos(0.95*winX,ICON_SIZE*winY*(1+(float)iconNum));
+					mouseIcons.setPos(0.90*winX,ICON_SIZE*winY*(1+(float)iconNum));
+				}
+
+				binderIcons.draw();
+				mouseIcons.draw();
+
+				if(foundKeyTex)
+					keyIcons.draw();
+
+				iconNum++;
+			}
+			
+		}
+
+		glPopAttrib();
+
+	}
+
+
+	glDisable(GL_ALPHA_TEST);
+	glEnable(GL_DEPTH_TEST);
+}
+
 void Scene::commitTempCam()
 {
 	ASSERT(tempCam);
@@ -477,9 +635,27 @@ void Scene::computeSceneLimits()
 {
 	boundCube.setInverseLimits();
 
-	if(!objects.size() && !refObjects.size())
+	BoundCube b;
+	for(unsigned int ui=0; ui<objects.size(); ui++)
 	{
-		//He's going to spend the rest of its life
+		objects[ui]->getBoundingBox(b);
+	
+		if(b.isValid())	
+			boundCube.expand(b);
+	}
+
+	for(unsigned int ui=0; ui<refObjects.size(); ui++)
+	{
+		refObjects[ui]->getBoundingBox(b);
+		
+		if(b.isValid())	
+			boundCube.expand(b);
+	}
+
+
+	if(!boundCube.isValid())
+	{
+		//He's going to spend the rest of his life
 		//in a one by one unit box.
 
 		//If there are no objects, then set the bounds
@@ -487,25 +663,6 @@ void Scene::computeSceneLimits()
 		boundCube.setBounds(-0.5,-0.5,-0.5,
 					0.5,0.5,0.5);
 	}
-	else
-	{
-		
-		BoundCube b;
-		for(unsigned int ui=0; ui<objects.size(); ui++)
-		{
-			objects[ui]->getBoundingBox(b);
-			
-			boundCube.expand(b);
-		}
-
-		for(unsigned int ui=0; ui<refObjects.size(); ui++)
-		{
-			refObjects[ui]->getBoundingBox(b);
-			
-			boundCube.expand(b);
-		}
-	}
-
 	//NOw that we have a scene level bounding box,
 	//we need to set the camera to ensure that
 	//this box is visible
@@ -650,16 +807,13 @@ void Scene::finaliseCam()
 {
 	switch(cameras[activeCam]->type())
 	{
-		case CAM_PERSP:
-			((CameraPerspective *)cameras[activeCam])->recomputeUpDirection();
-			break;
 		case CAM_LOOKAT:
-			((CameraPerspLookAt *)cameras[activeCam])->recomputeUpDirection();
+			((CameraLookAt *)cameras[activeCam])->recomputeUpDirection();
 			break;
 	}
 }
 
-void Scene::addSelectionDevices(const vector<SelectionDevice *> &d)
+void Scene::addSelectionDevices(const vector<SelectionDevice<Filter> *> &d)
 {
 	for(unsigned int ui=0;ui<d.size();ui++)
 	{
@@ -723,69 +877,51 @@ void Scene::applyDevice(float startX, float startY, float curX, float curY,
 	float viewWidth;
 	switch(cam->type())
 	{
-		case CAM_PERSP:
-			viewWidth=depth*tan(((CameraPerspective*)cam)->getFOV()/2.0f*M_PI/180.0);
-			break;
 		case CAM_LOOKAT:
-			viewWidth=depth*tan(((CameraPerspLookAt*)cam)->getFOV()/2.0f*M_PI/180.0);
-			break;
-		case CAM_FREE:
-			viewWidth=1.0;
+			viewWidth=((CameraLookAt*)cam)->getViewWidth(depth);
 			break;
+		default:
+			ASSERT(false);
 	}
 
 
 	//We have the object number, but we don't know which binding
 	//corresponds to this object. Search all bindings. It may be that more than one 
 	//binding is enabled for this object
-	unsigned int bindingStart=0;
-	bool bindingApplied=false;
 	SelectionBinding *binder;
-	while(bindingStart< selectionDevices.size())
+
+	vector<SelectionBinding*> activeBindings;
+	for(unsigned int ui=0;ui<selectionDevices.size();ui++)
 	{
+		if(selectionDevices[ui]->getBinding(
+			objects[lastSelected],mouseFlags,keyFlags,binder))
+			activeBindings.push_back(binder);
+	}
 
-		//Search for a binding
-		bool haveBinding;
-		haveBinding=false;
-
-		for(unsigned int ui=bindingStart;ui<selectionDevices.size();ui++)
-		{
-			if(selectionDevices[ui]->getBinding(
-				objects[lastSelected],mouseFlags,keyFlags,binder))
-			{
-				haveBinding=true;
-				break;
-			}
-		}
-
-		//If we didn't find one, abort
-		if(!haveBinding)
-			break;
-
+	for(unsigned int ui=0;ui<activeBindings.size();ui++)
+	{
 		//Convert the mouse-XY coords into a world coordinate, depending upon mouse/
 		//key cobinations
 		Point3D worldVec;
 		Point3D vectorCoeffs[2];
-		binder->computeWorldVectorCoeffs(mouseFlags,keyFlags,
+		activeBindings[ui]->computeWorldVectorCoeffs(mouseFlags,keyFlags,
 					vectorCoeffs[0],vectorCoeffs[1]);	
 
 		//Apply vector coeffs, dependant upon binding
-		worldVec = acrossDir*vectorCoeffs[0][0]*(curX-startX)
-				+ upDir*vectorCoeffs[0][1]*(curX-startX)
-				+ forwardsDir*vectorCoeffs[0][2]*(curX-startX)
+		worldVec = acrossDir*vectorCoeffs[0][0]*(curX-startX)*outWinAspect
+				+ upDir*vectorCoeffs[0][1]*(curX-startX)*outWinAspect
+				+ forwardsDir*vectorCoeffs[0][2]*(curX-startX)*outWinAspect
 				+ acrossDir*vectorCoeffs[1][0]*(curY-startY)
 				+ upDir*vectorCoeffs[1][1]*(curY-startY)
 				+ forwardsDir*vectorCoeffs[1][2]*(curY-startY);
 		worldVec*=viewWidth;
 		
-		binder->applyTransform(worldVec,permanent);
-		bindingApplied=true;
-		bindingStart++;
-
+		activeBindings[ui]->applyTransform(worldVec,permanent);
 	}
 
+	computeSceneLimits();
 	//Inform viscontrol about updates, if we have applied any
-	if(bindingApplied && permanent)
+	if(activeBindings.size() && permanent)
 	{
 		visControl->setUpdates();
 		//If the viscontrol is in the midddle of an update,
@@ -806,6 +942,14 @@ unsigned int Scene::duplicateCameras(vector<Camera *> &cams) const
 	return activeCam;
 }	
 
+void Scene::getEffects(vector<const Effect *> &eff) const
+{
+	eff.resize(effects.size());
+
+	for(unsigned int ui=0;ui<effects.size();ui++)	
+		eff[ui]=effects[ui];
+}	
+
 unsigned int Scene::getActiveCamId() const
 {
 	return camIDs.getId(activeCam);
@@ -834,10 +978,43 @@ bool Scene::isDefaultCam() const
 	return  activeCam==0;
 }
 
-void Scene::applyGLExtentions()
+bool Scene::camNameExists(const std::string &s) const
 {
-	const float glPointParams[3] = { 1.0, 0.5, 0.0 };
-#ifdef GL_VERSION_1_4
-	 glPointParameterfv(GL_DISTANCE_ATTENUATION_EXT, glPointParams);
-#endif
+	for(unsigned int ui=0;ui<cameras.size() ;ui++)
+	{
+		if(cameras[ui]->getUserString() == s)
+			return true;
+	}
+
+	return false;
+}
+
+
+
+unsigned int Scene::addEffect(Effect *e)
+{
+	ASSERT(e);
+	ASSERT(effects.size() == effectIDs.size());
+	effects.push_back(e);
+	
+
+	return effectIDs.genId(effects.size()-1);
+}
+
+
+void Scene::removeEffect(unsigned int uniqueID)
+{
+	unsigned int position = effectIDs.getPos(uniqueID);
+	delete effects[position];
+	effects.erase(effects.begin()+position);
+	effectIDs.killByPos(position);
+}
+
+void Scene::clearEffects()
+{
+	for(size_t ui=0;ui<effects.size();ui++)
+		delete effects[ui];
+
+	effects.clear();
+	effectIDs.clear();
 }
