@@ -17,6 +17,10 @@
 */
 
 #include "wxcomponents.h"
+#include "wxcommon.h"
+#include "basics.h"
+
+
 #include <wx/clipbrd.h>
 #include <wx/filefn.h>
 
@@ -26,9 +30,49 @@
 using std::ofstream;
 using std::vector;
 
+#ifdef __APPLE__
+const float FONT_HEADING_SCALEFACTOR=1.0f;
+#else
+const float FONT_HEADING_SCALEFACTOR=0.75f;
+#endif
 
 
 
+//Convert my internal choice string format to wx's
+std::string wxChoiceParamString(std::string choiceString)
+{
+	std::string retStr;
+
+	bool haveColon=false;
+	bool haveBar=false;
+	for(unsigned int ui=0;ui<choiceString.size();ui++)
+	{
+		if(haveColon && haveBar )
+		{
+			if(choiceString[ui] != ',')
+				retStr+=choiceString[ui];
+			else
+			{
+				haveBar=false;
+				retStr+=",";
+			}
+		}
+		else
+		{
+			if(choiceString[ui]==':')
+				haveColon=true;
+			else
+			{
+				if(choiceString[ui]=='|')
+					haveBar=true;
+			}
+		}
+	}
+
+	return retStr;
+}
+
+
 BEGIN_EVENT_TABLE(PropGridHandler, wxEvtHandler)
 	EVT_MOUSE_EVENTS(PropGridHandler::OnMouseMovement)
 END_EVENT_TABLE()
@@ -41,39 +85,6 @@ END_EVENT_TABLE()
 
 void PropGridHandler::OnMouseMovement(wxMouseEvent& evt)
 {
-/* FIXME: This is currently broken :( It is supposed to use the fact that the
- * combo box blocks mouse movement to determine that the gridCellChoiceEditor should defocus
- * when the user moves away from the box. However, I do not know how to post an event back
- * to threeDepict.cppp EVT_GRID_CMD_CELL_CHANGE(..), this doesn't automagically happen.
- * Easiest solution would be if upstream fixed it.
-	using std::cerr;
-	using std::endl;
-
-	cerr << "Mouse." << endl;
-	if (m_Grid->isComboEditing() && !eventGrace--)
-	{
-		cerr << "Is combo editing!" << endl;
-		wxGridCellEditor *ed;
-		ed = m_Grid->GetCellEditor(m_Grid->GetGridCursorRow(),m_Grid->GetGridCursorCol());
-		if(ed->IsCreated())
-		{
-			wxGridCellChoiceEditor *edChoice=((wxGridCellChoiceEditor *)ed);
-			cerr << "Ending edit" << endl;
-			//Modify the  
-			m_Grid->GetTable()->SetValue(m_Grid->GetGridCursorRow(),
-						m_Grid->GetGridCursorCol(),edChoice->GetValue());
-
-			//Force a focus move, which forces the editor to edit (but won't generate 
-			//an event :/ )
-//			m_Grid->SetGridCursor(0,1);
-			ed->evtHandler->SetInSetFocus(false);
-
-		}
-		eventGrace=5;
-
-	}
-
- */
 	evt.Skip();
 }
 
@@ -246,6 +257,7 @@ unsigned int wxPropertyGrid::getTypeFromRow(int row) const
 void  wxPropertyGrid::clearKeys()
 {
 	propertyKeys.clear();
+	sectionNames.clear();
 }
 
 void wxPropertyGrid::addKey(const std::string &name, unsigned int set,
@@ -375,13 +387,24 @@ void wxPropertyGrid::propertyLayout()
 		}
 
 		//Add a blank, light grey, read only row, but not if it is the last
-		//use to denote spacing between sets to user
+		//use to denote spacing between sets to user.
 		if (ui+1 < propertyKeys.size())
 		{
+			//Optionally give it a description; if one has been provided
+			if(sectionNames[ui+1].size())
+			{
+				this->SetCellValue(sepRows[ui],0,wxStr(sectionNames[ui+1]));
+				wxFont f;
+				f.SetStyle(wxFONTSTYLE_ITALIC);
+				f.SetPointSize(f.GetPointSize()*FONT_HEADING_SCALEFACTOR);
+				this->SetCellFont(sepRows[ui],0,f);
+			}
+
 			wxGridCellAttr *readOnlyRowAttr=new wxGridCellAttr;
 			readOnlyRowAttr->SetReadOnly(true);
 			readOnlyRowAttr->SetBackgroundColour(wxColour(*wxLIGHT_GREY));
 			this->SetRowAttr(sepRows[ui],readOnlyRowAttr);
+
 		}
 	}
 
@@ -532,10 +555,6 @@ void CopyGrid::saveData()
 
 void CopyGrid::OnKey(wxKeyEvent &event)
 {
-	if(event.MetaDown())
-	{
-		std::cerr << "Meta down" << std::endl;
-	}
 
 
 
@@ -589,9 +608,6 @@ void CopyGrid::copyData()
 	{
             for(int  c=0; c<cols; c++)
 	    {
-		int rVal,cVal;
-		rVal = coordTL.GetRow() + r;
-		cVal = coordTL.GetCol()+ c;
                 data+= stlStr(GetCellValue(coordTL.GetRow() + r,
 						coordTL.GetCol()+ c));
                 if(c < cols - 1)
